@@ -1,6 +1,9 @@
+import json
 import os
+from base64 import b64decode
 from datetime import datetime as Datetime
 from importlib.metadata import version
+from pathlib import Path
 
 from flask import Flask, Response, g, make_response, render_template, request
 
@@ -16,14 +19,16 @@ import chellow.bank_holidays
 import chellow.dloads
 import chellow.e.bmarketidx
 import chellow.e.bsuos
+import chellow.e.elexon
 import chellow.e.hh_importer
 import chellow.e.rcrc
 import chellow.e.system_price
-import chellow.e.tlms
 import chellow.e.views
+import chellow.fake_batch_updater
 import chellow.gas.cv
 import chellow.gas.views
 import chellow.national_grid
+import chellow.rrun
 import chellow.testing
 from chellow.models import (
     Contract,
@@ -50,22 +55,22 @@ TEMPLATE_FORMATS = {
 
 def get_importer_modules():
     return (
-        chellow.e.rcrc,
+        chellow.e.elexon,
         chellow.e.bsuos,
-        chellow.e.system_price,
-        chellow.e.hh_importer,
+        chellow.e.lcc,
         chellow.testing,
-        chellow.e.tlms,
         chellow.bank_holidays,
         chellow.gas.cv,
         chellow.e.bmarketidx,
         chellow.national_grid,
         chellow.rate_server,
+        chellow.rrun,
+        chellow.fake_batch_updater,
     )
 
 
-def create_app(testing=False):
-    app = Flask("chellow", instance_relative_config=True)
+def create_app(testing=False, instance_path=None):
+    app = Flask("chellow", instance_relative_config=True, instance_path=instance_path)
     app.wsgi_app = MsProxy(app.wsgi_app)
     app.secret_key = os.urandom(24)
     start_sqlalchemy()
@@ -73,14 +78,13 @@ def create_app(testing=False):
     app.register_blueprint(chellow.views.home)
     app.register_blueprint(chellow.e.views.e)
     app.register_blueprint(chellow.gas.views.gas)
-    chellow.utils.root_path = app.root_path
 
     api = chellow.api.api
     api.init_app(app, endpoint="/api/v1")
 
     if not testing:
         db_upgrade(app.root_path)
-        chellow.dloads.startup(app.instance_path)
+        chellow.e.hh_importer.startup()
 
         with Session() as sess:
             configuration = sess.execute(
@@ -90,6 +94,8 @@ def create_app(testing=False):
             api_props = props.get("api", {})
             api.description = api_props.get("description", "Access Chellow data")
 
+    chellow.dloads.startup(Path(app.instance_path), run_deleter=(not testing))
+
     for module in get_importer_modules():
         if not testing:
             module.startup()
@@ -113,20 +119,6 @@ def create_app(testing=False):
         )
         print(msg)
 
-        try:
-            scheme = request.headers["X-Forwarded-Proto"]
-        except KeyError:
-            config_contract = Contract.get_non_core_by_name(g.sess, "configuration")
-            props = config_contract.make_properties()
-            scheme = props.get("redirect_scheme", "http")
-
-        try:
-            host = request.headers["X-Forwarded-Host"]
-        except KeyError:
-            host = request.host
-
-        chellow.utils.url_root = scheme + "://" + host
-
     @app.before_request
     def check_permissions(*args, **kwargs):
         g.user = None
@@ -137,17 +129,45 @@ def create_app(testing=False):
             g.config = {}
         ad_props = g.config.get("ad_authentication", {})
         ad_auth_on = ad_props.get("on", False)
+        easy_auth_props = g.config.get("easy_auth", {})
+        easy_auth_on = easy_auth_props.get("on", False)
+        path = request.path
+        if path in ("/health", "/robots933456.txt"):
+            return
         if ad_auth_on:
             username = request.headers["X-Isrw-Proxy-Logon-User"].upper()
-            user = g.sess.query(User).filter(User.email_address == username).first()
+            user = g.sess.scalars(
+                select(User).where(User.email_address == username)
+            ).first()
             if user is None:
                 try:
                     username = ad_props["default_user"]
-                    user = (
-                        g.sess.query(User)
-                        .filter(User.email_address == username)
-                        .first()
-                    )
+                    user = g.sess.scalars(
+                        select(User).where(User.email_address == username)
+                    ).first()
+                except KeyError:
+                    user = None
+            if user is not None:
+                g.user = user
+        elif easy_auth_on:
+            jwt_enc = request.headers["X-MS-CLIENT-PRINCIPAL"]
+            jwt = json.loads(b64decode(jwt_enc))
+            for claim in jwt["claims"]:
+                if claim["typ"] == (
+                    "http://schemas.xmlsoap.org/ws/2005/05/identity/claims/"
+                    "emailaddress"
+                ):
+                    username = claim["val"]
+                    user = g.sess.scalars(
+                        select(User).where(User.email_address == username)
+                    ).first()
+                    break
+            if user is None:
+                try:
+                    username = easy_auth_props["default_user"]
+                    user = g.sess.scalars(
+                        select(User).where(User.email_address == username)
+                    ).first()
                 except KeyError:
                     user = None
             if user is not None:
@@ -166,25 +186,20 @@ def create_app(testing=False):
                         key = None
 
                     email = ips[key]
-                    g.user = (
-                        g.sess.query(User).filter(User.email_address == email).first()
-                    )
+                    g.user = g.sess.scalars(
+                        select(User).where(User.email_address == email)
+                    ).first()
                 except KeyError:
                     pass
             else:
-                user = (
-                    g.sess.query(User)
-                    .filter(User.email_address == auth.username)
-                    .first()
-                )
+                user = g.sess.scalars(
+                    select(User).where(User.email_address == auth.username)
+                ).first()
                 if user is not None and user.password_matches(auth.password):
                     g.user = user
 
         # Got our user
-        path = request.path
         method = request.method
-        if path in ("/health",):
-            return
 
         if g.user is not None:
             if "X-Isrw-Proxy-Logon-User" in request.headers:
@@ -201,6 +216,7 @@ def create_app(testing=False):
                     method in ("GET", "HEAD")
                     or path
                     in (
+                        "/reports/59",
                         "/reports/169",
                         "/reports/183",
                         "/reports/187",
@@ -226,7 +242,7 @@ def create_app(testing=False):
                         ):
                             return
                     elif market_role_code == "X":
-                        if path.startswith("/e/supplier_contracts/" + party.id):
+                        if path.startswith(f"/e/supplier_contracts/{party.id}"):
                             return
 
         if g.user is None and g.sess.query(User).count() == 0:
@@ -236,7 +252,9 @@ def create_app(testing=False):
             g.sess.commit()
             return
 
-        if g.user is None or (not ad_auth_on and auth is None):
+        if (not easy_auth_on) and (
+            (g.user is None) or (not ad_auth_on and auth is None)
+        ):
             return Response(
                 "Could not verify your access level for that URL.\n"
                 "You have to login with proper credentials",
@@ -262,14 +280,8 @@ def create_app(testing=False):
                 except NotFound:
                     pass
 
-        for importer in (
-            chellow.e.bsuos.bsuos_importer,
-            chellow.gas.cv.cv_importer,
-            chellow.e.bmarketidx.bmarketidx_importer,
-            chellow.national_grid.importer,
-            chellow.rate_server.importer,
-            chellow.testing.tester,
-        ):
+        for mod in get_importer_modules():
+            importer = mod.get_importer()
             if importer is not None and importer.global_alert is not None:
                 global_alerts.append(importer.global_alert)
 
