@@ -1,6 +1,7 @@
 import atexit
 import collections
 import csv
+import json
 import threading
 import traceback
 from datetime import datetime as Datetime
@@ -9,7 +10,6 @@ from io import BytesIO
 
 from dateutil.relativedelta import relativedelta
 
-from pypdf import PdfReader
 
 import requests
 
@@ -18,12 +18,7 @@ from sqlalchemy.sql.expression import null
 
 from zish import dumps, loads
 
-from chellow.models import (
-    Contract,
-    RateScript,
-    Session,
-    get_non_core_contract_id,
-)
+from chellow.models import Contract, RateScript, Session
 from chellow.national_grid import api_get
 from chellow.rate_server import download
 from chellow.utils import HH, ct_datetime, hh_format, to_ct, to_utc, utc_datetime_now
@@ -69,9 +64,8 @@ def hh(data_source, run="RF"):
             h["bsuos-rate"] = bsuos_rate = bsuos_cache[h["start-date"]]
         except KeyError:
             h_start = h["start-date"]
-            db_id = get_non_core_contract_id("bsuos")
 
-            rates = data_source.hh_rate(db_id, h_start)
+            rates = data_source.non_core_rate("bsuos", h_start)
 
             if h_start >= to_utc(ct_datetime(2023, 4, 1)):
                 try:
@@ -133,7 +127,7 @@ def _save_cache(sess, cache):
             del yr_cache[month]
 
 
-BASE_URL = "https://data.nationalgrideso.com/backend/dataset/"
+BASE_URL = "https://api.neso.energy/dataset/"
 PATHS = (
     "d6a4bf54-c63f-4014-a716-49fd3878ca52/resource/"
     "0eda5e28-1dc6-48da-8663-c00e12f2a1e2/download/current_ii_bsuos_data-11.csv",
@@ -223,8 +217,18 @@ def _process_url(logger, sess, url, contract, s):
     next(reader)  # Skip titles
 
     for row in reader:
-        date_str = row[0]
-        date_format = "%Y-%m-%dT%H:%M:%S" if len(date_str) == 19 else "%d-%m-%y"
+        date_str = row[0].strip()
+        len_date_str = len(date_str)
+        if len_date_str == 0:
+            continue
+        elif len_date_str == 19:
+            sep_char = date_str[13]
+            date_format = f"%Y-%m-%dT%H{sep_char}%M:%S"
+        elif len_date_str == 20:
+            date_format = "%Y-%m-%d-T%H:%M:%S"
+        else:
+            date_format = "%d-%m-%y"
+
         date = Datetime.strptime(date_str, date_format)
         period_str = row[1]
         period = int(period_str)
@@ -334,14 +338,8 @@ def shutdown():
 def find_rate(file_name, file_like, rate_index):
     rate_script = {"a_file_name": file_name}
     rates = []
-
-    reader = PdfReader(file_like)
-    for page in reader.pages:
-        for line in page.extract_text().splitlines():
-            if line.startswith("BSUoS Tariff £/MWh"):
-                for token in line.split():
-                    if token[0] == "£" and token[1] != "/":
-                        rates.append(Decimal(token[1:]))
+    for rate_str in json.load(file_like)["rates"]:
+        rates.append(Decimal(rate_str))
 
     rate_script["rate_gbp_per_mwh"] = rates[rate_index]
     return rate_script
@@ -392,11 +390,19 @@ def rate_server_import(sess, log, set_progress, s, paths):
 
             rs_1_script = rs_1.make_script()
             if rs_1_script.get("a_file_name") != file_name:
+                log(
+                    f"Found new file {file_name} for rate script starting "
+                    f"{hh_format(year_start)}"
+                )
                 rs_1.update(find_rate(file_name, BytesIO(download(s, url)), 0))
                 log(f"Updated BSUoS rate script for {hh_format(year_start)}")
 
             rs_2_script = rs_2.make_script()
             if rs_2_script.get("a_file_name") != file_name:
+                log(
+                    f"Found new file {file_name} for rate script starting "
+                    f"{hh_format(oct_start)}"
+                )
                 rs_2.update(find_rate(file_name, BytesIO(download(s, url)), 1))
                 log(f"Updated BSUoS rate script for {hh_format(oct_start)}")
 
@@ -407,7 +413,12 @@ def rate_server_import(sess, log, set_progress, s, paths):
 def national_grid_import(sess, log, set_progress, s):
     log("Starting to check for new BSUoS forecast")
 
-    contract = Contract.get_non_core_by_name(sess, "bsuos")
+    CONTRACT_NAME = "bsuos"
+    contract = Contract.find_non_core_by_name(sess, CONTRACT_NAME)
+    if contract is None:
+        contract = Contract.insert_non_core(
+            sess, CONTRACT_NAME, "", {}, to_utc(ct_datetime(1996, 4, 1)), None, {}
+        )
 
     block = None
 
