@@ -54,7 +54,7 @@ def get_rate(row, idx):
         return None
 
     val = cell.value
-    if val is None:
+    if val in (None, "-"):
         return None
     elif isinstance(val, str) and len(val) == 0:
         return None
@@ -145,6 +145,204 @@ BAND_WEEKEND = {
     "notes": None,
 }
 
+PERIODS = {
+    "monday to friday (including bank holidays) april to march inclusive": [
+        {
+            "weekend": False,
+            "start-month": 4,
+            "start-day": 1,
+            "finish-month": 12,
+            "finish-day": 31,
+        },
+        {
+            "weekend": False,
+            "start-month": 1,
+            "start-day": 1,
+            "finish-month": 3,
+            "finish-day": 31,
+        },
+    ],
+    "monday to friday (including bank holidays) april to october inclusive and march": [
+        {
+            "weekend": False,
+            "start-month": 4,
+            "start-day": 1,
+            "finish-month": 10,
+            "finish-day": 31,
+        },
+        {
+            "weekend": False,
+            "start-month": 3,
+            "start-day": 1,
+            "finish-month": 3,
+            "finish-day": 31,
+        },
+    ],
+    "monday to friday (including bank holidays) june to august inclusive": [
+        {
+            "weekend": False,
+            "start-month": 6,
+            "start-day": 1,
+            "finish-month": 8,
+            "finish-day": 31,
+        },
+    ],
+    "monday to friday (including bank holidays) november to february inclusive": [
+        {
+            "weekend": False,
+            "start-month": 11,
+            "start-day": 1,
+            "finish-month": 12,
+            "finish-day": 31,
+        },
+        {
+            "weekend": False,
+            "start-month": 1,
+            "start-day": 1,
+            "finish-month": 2,
+            "finish-day": "last",
+        },
+    ],
+    "monday to friday (including bank holidays) november to february inclusive "
+    "(excluding 22nd dec to 4th jan inclusive)": [
+        {
+            "weekend": False,
+            "start-month": 11,
+            "start-day": 1,
+            "finish-month": 12,
+            "finish-day": 21,
+        },
+        {
+            "weekend": False,
+            "start-month": 1,
+            "start-day": 5,
+            "finish-month": 2,
+            "finish-day": "last",
+        },
+    ],
+    "monday to friday (including bank holidays) march to october inclusive": [
+        {
+            "weekend": False,
+            "start-month": 3,
+            "start-day": 1,
+            "finish-month": 10,
+            "finish-day": 31,
+        },
+    ],
+    "monday to friday (including bank holidays) mar to oct inclusive": [
+        {
+            "weekend": False,
+            "start-month": 3,
+            "start-day": 1,
+            "finish-month": 10,
+            "finish-day": 31,
+        },
+    ],
+    "monday to friday (including bank holidays) nov to feb inclusive": [
+        {
+            "weekend": False,
+            "start-month": 11,
+            "start-day": 1,
+            "finish-month": 12,
+            "finish-day": 31,
+        },
+        {
+            "weekend": False,
+            "start-month": 1,
+            "start-day": 1,
+            "finish-month": 2,
+            "finish-day": "last",
+        },
+    ],
+    (
+        "monday to friday nov to feb (excluding 22nd dec to 4th jan inclusive)"
+        "monday to friday (including bank holidays) nov to feb inclusive "
+        "(excluding 22nd dec to 4th jan inclusive)"
+    ): [
+        {
+            "weekend": False,
+            "start-month": 11,
+            "start-day": 1,
+            "finish-month": 12,
+            "finish-day": 21,
+        },
+        {
+            "weekend": False,
+            "start-month": 1,
+            "start-day": 5,
+            "finish-month": 2,
+            "finish-day": "last",
+        },
+    ],
+    (
+        "monday to friday (including bank holidays) "
+        "mar to oct inclusive (plus 22nd dec to 4th jan inclusive)"
+    ): [
+        {
+            "weekend": False,
+            "start-month": 3,
+            "start-day": 1,
+            "finish-month": 10,
+            "finish-day": 31,
+        },
+        {
+            "weekend": False,
+            "start-month": 12,
+            "start-day": 22,
+            "finish-month": 12,
+            "finish-day": 31,
+        },
+        {
+            "weekend": False,
+            "start-month": 1,
+            "start-day": 1,
+            "finish-month": 1,
+            "finish-day": 4,
+        },
+    ],
+    (
+        "monday to friday "
+        "(including bank holidays) "
+        "nov to feb inclusive (excluding 22nd dec to 4th jan inclusive)"
+    ): [
+        {
+            "weekend": False,
+            "start-month": 11,
+            "start-day": 1,
+            "finish-month": 12,
+            "finish-day": 21,
+        },
+        {
+            "weekend": False,
+            "start-month": 1,
+            "start-day": 4,
+            "finish-month": 2,
+            "finish-day": "last",
+        },
+    ],
+    "saturday and sunday all year": [
+        {
+            "weekend": True,
+            "start-month": 1,
+            "start-day": 1,
+            "finish-month": 12,
+            "finish-day": 31,
+        }
+    ],
+    (
+        "monday to friday (including bank holidays) november to february "
+        "monday to friday nov to feb"
+    ): [
+        {
+            "weekend": False,
+            "start-month": 11,
+            "start-day": 1,
+            "finish-month": 2,
+            "finish-day": "last",
+        }
+    ],
+}
+
 
 def str_to_hr(hr_str):
     for sep in (":", "."):
@@ -180,7 +378,7 @@ def val_to_slots(val):
     return slots
 
 
-def col_match(row, pattern, repeats=1):
+def col_find(row, pattern, repeats=1):
     for i, cell in enumerate(row):
         txt = cell.value
         if txt is not None:
@@ -191,10 +389,15 @@ def col_match(row, pattern, repeats=1):
                 else:
                     repeats -= 1
 
-    raise BadRequest(
-        f"Pattern '{pattern}' not found in row "
-        + ", ".join(str(cell.value) for cell in row)
-    )
+
+def col_match(row, pattern, repeats=1):
+    result = col_find(row, pattern, repeats=repeats)
+    if result is None:
+        raise BadRequest(
+            f"Pattern '{pattern}' not found in row "
+            + ", ".join(str(cell.value) for cell in row)
+        )
+    return result
 
 
 def tab_lv_hv(sheet, gsp_rates):
@@ -204,6 +407,7 @@ def tab_lv_hv(sheet, gsp_rates):
         tariffs = gsp_rates["tariffs"] = {}
 
     bands = gsp_rates["bands"] = []
+    ums_bands = gsp_rates["ums_bands"] = []
 
     class State(Enum):
         OUTSIDE = auto()
@@ -231,15 +435,31 @@ def tab_lv_hv(sheet, gsp_rates):
 
             if is_weekend is None:
                 state = State.OUTSIDE
-            else:
-                for i, band_name in enumerate(("red", "amber")):
-                    for slot in val_to_slots(get_value(row, i + 1)):
-                        bands.append(
+                continue
+
+            for i, band_name in enumerate(("red", "amber")):
+                for slot in val_to_slots(get_value(row, i + 1)):
+                    bands.append(
+                        {
+                            "weekend": is_weekend,
+                            "start": slot["start"],
+                            "finish": slot["finish"],
+                            "band": band_name,
+                        }
+                    )
+
+            val_6 = " ".join(get_value(row, 6).split()).lower()
+            periods = PERIODS[val_6]
+
+            for i, band_name in enumerate(("black", "yellow")):
+                for slot in val_to_slots(get_value(row, i + 8)):
+                    for period in periods:
+                        ums_bands.append(
                             {
-                                "weekend": is_weekend,
-                                "start": slot["start"],
-                                "finish": slot["finish"],
                                 "band": band_name,
+                                **period,
+                                "start-decimal-hour": slot["start"],
+                                "finish-decimal-hour": slot["finish"],
                             }
                         )
 
@@ -250,6 +470,9 @@ def tab_lv_hv(sheet, gsp_rates):
                 llfcs_str = ",".join(
                     chain(to_llfcs(get_value(row, 1)), to_llfcs(get_value(row, 10)))
                 )
+                is_ums = "unmetered supplies" in val_0.lower()
+                high_band = "black" if is_ums else "red"
+                medium_band = "yellow" if is_ums else "amber"
                 pcs_str = ",".join(to_pcs(row, 2))
                 tariffs[llfcs_str + "_" + pcs_str] = {
                     "description": val_0,
@@ -262,8 +485,10 @@ def tab_lv_hv(sheet, gsp_rates):
                     "excess-gbp-per-kva-per-day": get_zero_rate(
                         row, col_match(title_row, "exce")
                     ),
-                    "red-gbp-per-kwh": get_rag_rate(row, col_match(title_row, "red")),
-                    "amber-gbp-per-kwh": get_rag_rate(
+                    f"{high_band}-gbp-per-kwh": get_rag_rate(
+                        row, col_match(title_row, "red")
+                    ),
+                    f"{medium_band}-gbp-per-kwh": get_rag_rate(
                         row, col_match(title_row, "amber")
                     ),
                     "green-gbp-per-kwh": get_rag_rate(
@@ -308,19 +533,30 @@ def tab_ehv(sheet, gsp_rates):
                 llfc_val = get_value(row, col_match(title_row, "llfc", repeats=repeats))
                 llfc = None if llfc_val is None else str(llfc_val).strip()
                 if llfc not in (None, ""):
+                    band_col = col_find(title_row, "residual")
+                    if band_col is None:
+                        band = 0
+                    else:
+                        band_val = get_value(row, band_col)
+                        if band_val in (None, ""):
+                            band = 0
+                        else:
+                            band = int(band_val)
+
                     tariffs[llfc] = {
-                        "gbp-per-kwh": get_rate(
-                            row, col_match(title_row, polarity + " super red")
+                        "super-red-gbp-per-kwh": get_rate(
+                            row, col_match(title_row, f"{polarity} super red")
                         ),
-                        "gbp-per-day": get_zero_rate(
-                            row, col_match(title_row, polarity + " fixed")
+                        "gbp-per-mpan-per-day": get_zero_rate(
+                            row, col_match(title_row, f"{polarity} fixed")
                         ),
                         "gbp-per-kva-per-day": get_zero_rate(
-                            row, col_match(title_row, polarity + " capacity")
+                            row, col_match(title_row, f"{polarity} capacity")
                         ),
                         "excess-gbp-per-kva-per-day": get_zero_rate(
-                            row, col_match(title_row, polarity + " exce")
+                            row, col_match(title_row, f"{polarity} exce")
                         ),
+                        "description": f"Designated EHV{band}",
                     }
 
         elif state == EHV_BANDS:
@@ -328,53 +564,15 @@ def tab_ehv(sheet, gsp_rates):
                 state = EHV_BLANK
             else:
                 period_str = " ".join(get_value(row, 0).split()).lower()
-                periods = []
-
-                if (
-                    period_str
-                    == "monday to friday nov to feb (excluding "
-                    + "22nd dec to 4th jan inclusive)"
-                ):
-                    periods.append(
-                        {
-                            "weekend": False,
-                            "start-month": 11,
-                            "start-day": 1,
-                            "finish-month": 12,
-                            "finish-day": 21,
-                        }
-                    )
-                    periods.append(
-                        {
-                            "weekend": False,
-                            "start-month": 1,
-                            "start-day": 5,
-                            "finish-month": 2,
-                            "finish-day": "last",
-                        }
-                    )
-
-                elif period_str in (
-                    "monday to friday (including bank holidays) november to february",
-                    "monday to friday nov to feb",
-                ):
-                    periods.append(
-                        {
-                            "weekend": False,
-                            "start-month": 11,
-                            "start-day": 1,
-                            "finish-month": 2,
-                            "finish-day": "last",
-                        }
-                    )
+                periods = PERIODS[period_str]
 
                 for slot in val_to_slots(get_value(row, 3)):
                     for period in periods:
                         bands.append(
                             {
                                 **period,
-                                "start_hour": slot["start"],
-                                "finish_hour": slot["finish"],
+                                "start-decimal-hour": slot["start"],
+                                "finish-decimal-hour": slot["finish"],
                             }
                         )
 
@@ -409,19 +607,27 @@ def find_rates(file_name, file_like):
 
 
 GSP_MAP = (
+    ("eastern-power-networks-", "_A"),
     ("gsp a", "_A"),
     ("gsp b", "_B"),
     ("gsp c", "_C"),
+    ("lond", "_C"),
     ("gsp d", "_D"),
+    ("spm-", "_D"),
     ("gsp e", "_E"),
+    ("mide", "_E"),
     ("gsp f", "_F"),
+    ("enwl-", "_G"),
     ("gsp g", "_G"),
     ("gsp h", "_H"),
+    ("sepd", "_H"),
     ("gsp j", "_J"),
     ("gsp k", "_K"),
     ("gsp l", "_L"),
+    ("sweb", "_L"),
     ("gsp m", "_M"),
     ("gsp n", "_N"),
+    ("spd", "_N"),
     ("gsp p", "_P"),
     ("gsp_a", "_A"),
     ("gsp_b", "_B"),
@@ -437,10 +643,6 @@ GSP_MAP = (
     ("gsp_m", "_M"),
     ("gsp_n", "_N"),
     ("gsp_p", "_P"),
-    ("lond", "_C"),
-    ("mide", "_E"),
-    ("sepd", "_H"),
-    ("sweb", "_L"),
 )
 
 
@@ -575,12 +777,20 @@ VL_LOOKUP = {
     "greater than 22kv connected - demand": HV_NET,
     "greater than 22kv connected - generation": HV_NET,
     "high voltage network": HV_NET,
+    "high voltage network export": HV_NET,
+    "high voltage network import": HV_NET,
     "high-voltage network": HV_NET,
     "high voltage substation": HV_SUB,
+    "high voltage substation export": HV_SUB,
+    "high voltage substation import": HV_SUB,
     "high-voltage substation": HV_SUB,
     "low voltage network": LV_NET,
+    "low voltage network export": LV_NET,
+    "low voltage network import": LV_NET,
     "low-voltage network": LV_NET,
     "low voltage substation": LV_SUB,
+    "low voltage substation export": LV_SUB,
+    "low voltage substation import": LV_SUB,
     "low-voltage substation": LV_SUB,
 }
 
@@ -626,15 +836,15 @@ def update_vls(sess, logger, vls, dno_code, fy_start, rs_finish):
         llfc = sess.execute(q).scalar_one_or_none()
 
         if llfc is None:
-            raise BadRequest(
+            logger(
                 f"There is no LLFC with the code '{vl_code}' associated with the DNO "
-                f"{dno.code} from {hh_format(fy_start)} to {hh_format(rs_finish)}."
+                f"{dno.dno_code} from {hh_format(fy_start)} to {hh_format(rs_finish)}."
             )
+        else:
+            vl_voltage_level = VoltageLevel.get_by_code(sess, vl["voltage_level"])
+            llfc.voltage_level = vl_voltage_level
 
-        vl_voltage_level = VoltageLevel.get_by_code(sess, vl["voltage_level"])
-        llfc.voltage_level = vl_voltage_level
-
-        llfc.is_substation = vl["is_substation"]
-        if sess.is_modified(llfc):
-            logger(f"Updated LLFC {llfc.code} of DNO {dno_code}")
-            sess.flush()
+            llfc.is_substation = vl["is_substation"]
+            if sess.is_modified(llfc):
+                logger(f"Updated LLFC {llfc.code} of DNO {dno_code}")
+                sess.flush()
