@@ -1,7 +1,7 @@
 from collections import defaultdict
 from datetime import timedelta
 from itertools import combinations, count
-from math import floor, log10
+from math import log10
 from types import MappingProxyType
 
 from dateutil.relativedelta import relativedelta
@@ -13,7 +13,6 @@ from werkzeug.exceptions import BadRequest
 
 from zish import dumps, loads
 
-import chellow.bank_holidays
 from chellow.e.computer import hh_rate
 from chellow.models import (
     BillType,
@@ -24,7 +23,6 @@ from chellow.models import (
     GRateScript,
     GReadType,
     GRegisterRead,
-    get_g_industry_contract_id,
 )
 from chellow.utils import (
     HH,
@@ -77,8 +75,14 @@ def get_g_engine_cache(caches, name):
     try:
         return caches["g_engine"][name]
     except KeyError:
-        caches["g_engine"] = defaultdict(dict)
-        return caches["g_engine"][name]
+        try:
+            gcache = caches["g_engine"]
+        except KeyError:
+            gcache = caches["g_enine"] = {}
+
+        cache = gcache[name] = {}
+
+        return cache
 
 
 def g_contract_func(caches, contract, func_name):
@@ -206,6 +210,8 @@ def datum_range(sess, caches, years_back, start_date, finish_date):
                         "unit_factor": 1,
                         "units_consumed": 0,
                         "correction_factor": 1,
+                        "aq": 0,
+                        "soq": 0,
                         "calorific_value": 0,
                         "avg_cv": 0,
                     }
@@ -215,15 +221,20 @@ def datum_range(sess, caches, years_back, start_date, finish_date):
         d_cache[finish_date] = datum_tuple
         return datum_tuple
 
-    def g_rates(self, g_contract_id_or_name, date):
-        return g_rates(self.sess, self.caches, g_contract_id_or_name, date)
-
 
 ACTUAL_READ_TYPES = ["A", "C", "S"]
 CORRECTION_FACTOR = 1.02264
 
 
-def g_rates(sess, caches, g_contract_id_or_name, date):
+def g_industry_rates(sess, caches, g_contract_id_or_name, date):
+    return g_rates(sess, caches, g_contract_id_or_name, date, True)
+
+
+def g_supplier_rates(sess, caches, g_contract_id_or_name, date):
+    return g_rates(sess, caches, g_contract_id_or_name, date, False)
+
+
+def g_rates(sess, caches, g_contract_id_or_name, date, is_industry):
     try:
         return caches["g_engine"]["rates"][g_contract_id_or_name][date]
     except KeyError:
@@ -238,17 +249,36 @@ def g_rates(sess, caches, g_contract_id_or_name, date):
             rss_cache = ccache["rates"] = {}
 
         try:
-            cont_cache = rss_cache[g_contract_id_or_name]
+            i_cache = rss_cache[is_industry]
         except KeyError:
-            cont_cache = rss_cache[g_contract_id_or_name] = {}
+            i_cache = rss_cache[is_industry] = {}
+
+        try:
+            cont_cache = i_cache[g_contract_id_or_name]
+        except KeyError:
+            cont_cache = i_cache[g_contract_id_or_name] = {}
 
         try:
             return cont_cache[date]
         except KeyError:
             if isinstance(g_contract_id_or_name, int):
-                g_contract = GContract.get_by_id(sess, g_contract_id_or_name)
+                if is_industry:
+                    g_contract = GContract.get_industry_by_id(
+                        sess, g_contract_id_or_name
+                    )
+                else:
+                    g_contract = GContract.get_supplier_by_id(
+                        sess, g_contract_id_or_name
+                    )
             elif isinstance(g_contract_id_or_name, str):
-                g_contract = GContract.get_by_name(sess, g_contract_id_or_name)
+                if is_industry:
+                    g_contract = GContract.get_industry_by_name(
+                        sess, g_contract_id_or_name
+                    )
+                else:
+                    g_contract = GContract.get_supplier_by_name(
+                        sess, g_contract_id_or_name
+                    )
             else:
                 raise BadRequest("g_contract_id_or_name must be an int or str")
 
@@ -292,8 +322,7 @@ def g_rates(sess, caches, g_contract_id_or_name, date):
             prefix = "industry" if g_contract.is_industry else "supplier"
 
             vals = PropDict(
-                f"the rate script {chellow.utils.url_root}/g/{prefix}_rate_scripts/"
-                f"{rs.id} ",
+                f"the rate script /g/{prefix}_rate_scripts/{rs.id} ",
                 loads(rs.script),
                 [],
             )
@@ -428,8 +457,6 @@ class GDataSource:
                     .all()
                 )
 
-        cv_id = get_g_industry_contract_id("cv")
-        ug_id = get_g_industry_contract_id("ug")
         hist_map = {}
 
         for i, hist_g_era in enumerate(hist_g_eras):
@@ -450,7 +477,6 @@ class GDataSource:
                     chunk_start,
                     chunk_finish,
                     hist_g_era,
-                    cv_id,
                     self.g_ldz_code,
                     hist_map,
                     forecast_date,
@@ -463,7 +489,6 @@ class GDataSource:
                     hist_g_era,
                     chunk_start,
                     chunk_finish,
-                    cv_id,
                     hist_map,
                     self.g_ldz_code,
                 )
@@ -481,21 +506,20 @@ class GDataSource:
                 * h["calorific_value"]
                 / 3.6
             )
-            h["kwh_avg"] = (
-                h["units_consumed"]
-                * h["unit_factor"]
-                * h["correction_factor"]
-                * h["avg_cv"]
-                / 3.6
-            )
             h["ug_rate"] = float(
-                g_rates(sess, self.caches, ug_id, h["start_date"])["ug_gbp_per_kwh"][
-                    self.g_exit_zone_code
-                ]
+                g_rates(sess, self.caches, "ug", h["start_date"], True)[
+                    "ug_gbp_per_kwh"
+                ][self.g_exit_zone_code]
             )
             self.hh_data.append(h)
             self.bill_hhs[d["start_date"]] = {}
 
+    def g_industry_rates(self, g_contract_id_or_name, date):
+        return g_industry_rates(self.sess, self.caches, g_contract_id_or_name, date)
+
+    def g_supplier_rates(self, g_contract_id_or_name, date):
+        return g_supplier_rates(self.sess, self.caches, g_contract_id_or_name, date)
+
 
 def _no_bill_kwh(
     sess,
@@ -504,7 +528,6 @@ def _no_bill_kwh(
     start,
     finish,
     g_era,
-    cv_id,
     g_ldz_code,
     hist_map,
     forecast_date,
@@ -533,7 +556,7 @@ def _no_bill_kwh(
                     break
 
     consumption_info = "read list - \n" + dumps(read_list) + "\n"
-    hhs = _find_hhs(sess, caches, g_era, pairs, start, finish, cv_id, g_ldz_code)
+    hhs = _find_hhs(sess, caches, g_era, pairs, start, finish, g_ldz_code)
     _set_status(hhs, read_list, forecast_date)
     hist_map.update(hhs)
     return consumption_info + "pairs - \n" + dumps(pairs)
@@ -564,26 +587,22 @@ def _find_pair(is_forwards, read_list):
 
 
 def _bill_kwh(
-    sess,
-    caches,
-    g_supply,
-    hist_g_era,
-    chunk_start,
-    chunk_finish,
-    cv_id,
-    hist_map,
-    g_ldz_code,
+    sess, caches, g_supply, hist_g_era, chunk_start, chunk_finish, hist_map, g_ldz_code
 ):
     cf = float(hist_g_era.correction_factor)
+    aq = float(hist_g_era.aq)
+    soq = float(hist_g_era.soq)
     g_unit = hist_g_era.g_unit
     unit_code, unit_factor = g_unit.code, float(g_unit.factor)
 
     for hh_date in hh_range(caches, chunk_start, chunk_finish):
-        cv, avg_cv = find_cv(sess, caches, cv_id, hh_date, g_ldz_code)
+        cv, avg_cv = find_cv(sess, caches, hh_date, g_ldz_code)
         hist_map[hh_date] = {
             "unit_code": unit_code,
             "unit_factor": unit_factor,
             "correction_factor": cf,
+            "aq": aq,
+            "soq": soq,
             "calorific_value": cv,
             "avg_cv": avg_cv,
         }
@@ -644,8 +663,8 @@ def _bill_kwh(
             hist_map[hh_date]["units_consumed"] = hh_units_consumed
 
 
-def find_cv(sess, caches, cv_id, dt, g_ldz_code):
-    cvs = g_rates(sess, caches, cv_id, dt)["cvs"][g_ldz_code]
+def find_cv(sess, caches, dt, g_ldz_code):
+    cvs = g_rates(sess, caches, "cv", dt, True)["cvs"][g_ldz_code]
     ct = to_ct(dt)
     try:
         cv_props = cvs[ct.day]
@@ -681,15 +700,11 @@ def find_cv(sess, caches, cv_id, dt, g_ldz_code):
             avg_cv = year_cache[ct.month]
         except KeyError:
             cv_list = [float(v["cv"]) for v in cvs.values()]
-            avg_cv = year_cache[ct.month] = (
-                floor((sum(cv_list) / len(cv_list)) * 10) / 10
-            )
+            avg_cv = year_cache[ct.month] = sum(cv_list) / len(cv_list)
     return cv, avg_cv
 
 
-def _find_hhs(
-    sess, caches, hist_g_era, pairs, chunk_start, chunk_finish, g_cv_id, g_ldz_code
-):
+def _find_hhs(sess, caches, hist_g_era, pairs, chunk_start, chunk_finish, g_ldz_code):
     hhs = {}
     if len(pairs) == 0:
         pairs.append({"start-date": chunk_start, "units": 0})
@@ -716,18 +731,22 @@ def _find_hhs(
         pairs[-1]["finish-date"] = chunk_finish
 
     cf = float(hist_g_era.correction_factor)
+    aq = float(hist_g_era.aq)
+    soq = float(hist_g_era.soq)
     g_unit = hist_g_era.g_unit
     unit_code, unit_factor = g_unit.code, float(g_unit.factor)
     for pair in pairs:
         units = pair["units"]
         for hh_date in hh_range(caches, pair["start-date"], pair["finish-date"]):
-            cv, avg_cv = find_cv(sess, caches, g_cv_id, hh_date, g_ldz_code)
+            cv, avg_cv = find_cv(sess, caches, hh_date, g_ldz_code)
 
             hhs[hh_date] = {
                 "unit_code": unit_code,
                 "unit_factor": unit_factor,
                 "units_consumed": units,
                 "correction_factor": cf,
+                "aq": aq,
+                "soq": soq,
                 "calorific_value": cv,
                 "avg_cv": avg_cv,
             }
