@@ -1,6 +1,7 @@
 import csv
 from collections import defaultdict
 from datetime import datetime as Datetime
+from decimal import Decimal
 from importlib import import_module
 from io import BytesIO, StringIO
 
@@ -12,16 +13,19 @@ from flask import (
     flash,
     g,
     make_response,
+    redirect,
     render_template as rtemplate,
     request,
 )
 
-from sqlalchemy import false, select, text, true
-from sqlalchemy.orm import joinedload
+from sqlalchemy import false, func, null, select, text, true
+from sqlalchemy.orm import aliased, joinedload
 
 
 from werkzeug.exceptions import BadRequest
 
+from zish import dumps, loads
+
 import chellow.gas.bill_import
 import chellow.gas.dn_rate_parser
 from chellow.models import (
@@ -47,7 +51,6 @@ from chellow.models import (
 from chellow.utils import (
     HH,
     csv_make_val,
-    hh_format,
     req_bool,
     req_date,
     req_decimal,
@@ -58,11 +61,11 @@ from chellow.utils import (
     utc_datetime,
     utc_datetime_now,
 )
-from chellow.views import chellow_redirect as credirect, hx_redirect as chx_redirect
+from chellow.views import hx_redirect as chx_redirect
 
 
 def chellow_redirect(path, code=None):
-    return credirect(f"/g{path}", code)
+    return redirect(f"/g{path}", code)
 
 
 def hx_redirect(path, code=None):
@@ -127,12 +130,11 @@ def supply_get(g_supply_id):
     try:
         g_era_bundles = []
         g_supply = GSupply.get_by_id(g.sess, g_supply_id)
-        g_eras = (
-            g.sess.query(GEra)
-            .filter(GEra.g_supply == g_supply)
+        g_eras = g.sess.scalars(
+            select(GEra)
+            .where(GEra.g_supply == g_supply)
             .order_by(GEra.start_date.desc())
-            .all()
-        )
+        ).all()
         for g_era in g_eras:
             physical_site = (
                 g.sess.query(Site)
@@ -168,20 +170,20 @@ def supply_get(g_supply_id):
             )
 
             g_bills = (
-                g.sess.query(GBill)
-                .filter(GBill.g_supply == g_supply)
+                select(GBill)
+                .where(GBill.g_supply == g_supply)
                 .order_by(
                     GBill.start_date.desc(),
                     GBill.issue_date.desc(),
                     GBill.reference.desc(),
                 )
             )
-            if g_era.finish_date is not None:
-                g_bills = g_bills.filter(GBill.start_date <= g_era.finish_date)
+            if g_era != g_eras[0]:
+                g_bills = g_bills.where(GBill.start_date < g_era.finish_date)
             if g_era != g_eras[-1]:
-                g_bills = g_bills.filter(GBill.start_date >= g_era.start_date)
+                g_bills = g_bills.where(GBill.start_date >= g_era.start_date)
 
-            for g_bill in g_bills:
+            for g_bill in g.sess.scalars(g_bills):
                 g_reads = (
                     g.sess.query(GRegisterRead)
                     .filter(GRegisterRead.g_bill == g_bill)
@@ -218,7 +220,7 @@ def supply_get(g_supply_id):
 
         RELATIVE_YEAR = relativedelta(years=1)
 
-        now = Datetime.utcnow()
+        now = utc_datetime_now()
         triad_year = (now - RELATIVE_YEAR).year if now.month < 3 else now.year
         this_month_start = Datetime(now.year, now.month, 1)
         last_month_start = this_month_start - relativedelta(months=1)
@@ -231,24 +233,19 @@ def supply_get(g_supply_id):
             for report_id in properties["g_supply_reports"]:
                 batch_reports.append(Report.get_by_id(g.sess, report_id))
 
-        truncated_note = None
-        is_truncated = False
-        note = None
-        if len(g_supply.note.strip()) == 0:
-            note_str = "{'notes': []}"
-        else:
-            note_str = g_supply.note
+        note = truncated_line = None
+        g_supply_note = (
+            {"notes": []} if len(g_supply.note.strip()) == 0 else loads(g_supply.note)
+        )
 
-        supply_note = eval(note_str)
-        notes = supply_note["notes"]
+        notes = g_supply_note["notes"]
         if len(notes) > 0:
-            note = notes[0]
+            note = notes[-1]
             lines = note["body"].splitlines()
             if len(lines) > 0:
-                trunc_line = lines[0][:50]
-                if len(lines) > 1 or len(lines[0]) > len(trunc_line):
-                    is_truncated = True
-                    truncated_note = trunc_line
+                line0 = lines[0]
+                if len(lines) > 1 or len(line0) > 50:
+                    truncated_line = line0[:50]
         return render_template(
             "supply.html",
             triad_year=triad_year,
@@ -258,8 +255,7 @@ def supply_get(g_supply_id):
             g_era_bundles=g_era_bundles,
             g_supply=g_supply,
             system_properties=properties,
-            is_truncated=is_truncated,
-            truncated_note=truncated_note,
+            truncated_line=truncated_line,
             note=note,
             this_month_start=this_month_start,
             batch_reports=batch_reports,
@@ -289,15 +285,37 @@ def supply_edit_get(g_supply_id):
     )
 
 
+@gas.route("/supplies/<int:g_supply_id>/edit", methods=["DELETE"])
+def supply_edit_delete(g_supply_id):
+    g_supply = GSupply.get_by_id(g.sess, g_supply_id)
+    try:
+        g_supply.delete(g.sess)
+        g.sess.commit()
+        return hx_redirect("/supplies")
+    except BadRequest as e:
+        flash(e.description)
+        g_eras = g.sess.scalars(
+            select(GEra)
+            .where(GEra.g_supply == g_supply)
+            .order_by(GEra.start_date.desc())
+        )
+        g_exit_zones = g.sess.query(GExitZone).order_by(GExitZone.code).all()
+        return make_response(
+            render_template(
+                "supply_edit.html",
+                g_supply=g_supply,
+                g_eras=g_eras,
+                g_exit_zones=g_exit_zones,
+            ),
+            400,
+        )
+
+
 @gas.route("/supplies/<int:g_supply_id>/edit", methods=["POST"])
 def supply_edit_post(g_supply_id):
     g_supply = GSupply.get_by_id(g.sess, g_supply_id)
     try:
-        if "delete" in request.values:
-            g_supply.delete(g.sess)
-            g.sess.commit()
-            return chellow_redirect("/supplies", 303)
-        elif "insert_g_era" in request.values:
+        if "insert_g_era" in request.values:
             start_date = req_date("start")
             g_supply.insert_g_era_at(g.sess, start_date)
             g.sess.commit()
@@ -333,9 +351,18 @@ def supply_edit_post(g_supply_id):
 def batches_get():
     g_contract_id = req_int("g_contract_id")
     g_contract = GContract.get_by_id(g.sess, g_contract_id)
-    g_batches = (
-        g.sess.query(GBatch)
-        .filter(GBatch.g_contract == g_contract)
+    g_batches = g.sess.execute(
+        select(
+            GBatch,
+            func.count(GBill.id),
+            func.coalesce(func.sum(GBill.net), 0),
+            func.coalesce(func.sum(GBill.vat), 0),
+            func.coalesce(func.sum(GBill.gross), 0),
+            func.coalesce(func.sum(GBill.kwh), 0),
+        )
+        .join(GBill, isouter=True)
+        .where(GBatch.g_contract == g_contract)
+        .group_by(GBatch.id)
         .order_by(GBatch.reference.desc())
     )
     return render_template("batches.html", g_contract=g_contract, g_batches=g_batches)
@@ -418,6 +445,16 @@ def batch_get(g_batch_id):
                 vbd["vat"] += g_bill.vat
                 vbd["net"] += g_bill.net
 
+            if "vat" in bd:
+                for vat_percentage, vat_bd in bd["vat"].items():
+                    try:
+                        vbd = vat_breakdown[vat_percentage]
+                    except KeyError:
+                        vbd = vat_breakdown[vat_percentage] = defaultdict(int)
+
+                    vbd["vat"] += vat_bd.get("vat", Decimal("0"))
+                    vbd["net"] += vat_bd.get("net", Decimal("0"))
+
     config_contract = Contract.get_non_core_by_name(g.sess, "configuration")
     properties = config_contract.make_properties()
 
@@ -688,10 +725,9 @@ def supply_notes_get(g_supply_id):
     g_supply = GSupply.get_by_id(g.sess, g_supply_id)
 
     if len(g_supply.note.strip()) > 0:
-        note_str = g_supply.note
+        g_supply_note = loads(g_supply.note)
     else:
-        note_str = "{'notes': []}"
-    g_supply_note = eval(note_str)
+        g_supply_note = {"notes": []}
 
     return render_template(
         "supply_notes.html", g_supply=g_supply, g_supply_note=g_supply_note
@@ -709,15 +745,13 @@ def supply_note_add_post(g_supply_id):
     try:
         g_supply = GSupply.get_by_id(g.sess, g_supply_id)
         body = req_str("body")
-        category = req_str("category")
-        is_important = req_bool("is_important")
         if len(g_supply.note.strip()) == 0:
-            g_supply.note = "{'notes': []}"
-        note_dict = eval(g_supply.note)
-        note_dict["notes"].append(
-            {"category": category, "is_important": is_important, "body": body}
-        )
-        g_supply.note = str(note_dict)
+            note_dict = {"notes": []}
+        else:
+            note_dict = loads(g_supply.note)
+
+        note_dict["notes"].append({"body": body, "timestamp": utc_datetime_now()})
+        g_supply.note = dumps(note_dict)
         g.sess.commit()
         return chellow_redirect(f"/supplies/{g_supply_id}", 303)
     except BadRequest as e:
@@ -730,36 +764,43 @@ def supply_note_add_post(g_supply_id):
 @gas.route("/supplies/<int:g_supply_id>/notes/<int:index>/edit")
 def supply_note_edit_get(g_supply_id, index):
     g_supply = GSupply.get_by_id(g.sess, g_supply_id)
-    g_supply_note = eval(g_supply.note)
+    g_supply_note = loads(g_supply.note)
     note = g_supply_note["notes"][index]
     note["index"] = index
     return render_template("supply_note_edit.html", g_supply=g_supply, note=note)
 
 
+@gas.route("/supplies/<int:g_supply_id>/notes/<int:index>/edit", methods=["DELETE"])
+def supply_note_edit_delete(g_supply_id, index):
+    try:
+        g_supply = GSupply.get_by_id(g.sess, g_supply_id)
+        g_supply_note = loads(g_supply.note)
+        del g_supply_note["notes"][index]
+        g_supply.note = dumps(g_supply_note)
+        g.sess.commit()
+        return hx_redirect(f"/supplies/{g_supply_id}/notes")
+    except BadRequest as e:
+        flash(e.description)
+        g_supply_note = loads(g_supply.note)
+        note = g_supply_note["notes"][index]
+        note["index"] = index
+        return render_template("supply_note_edit.html", g_supply=g_supply, note=note)
+
+
 @gas.route("/supplies/<int:g_supply_id>/notes/<int:index>/edit", methods=["POST"])
 def supply_note_edit_post(g_supply_id, index):
     try:
         g_supply = GSupply.get_by_id(g.sess, g_supply_id)
-        g_supply_note = eval(g_supply.note)
-        if "delete" in request.values:
-            del g_supply_note["notes"][index]
-            g_supply.note = str(g_supply_note)
-            g.sess.commit()
-            return chellow_redirect(f"/supplies/{g_supply_id}/notes", 303)
-        else:
-            category = req_str("category")
-            is_important = req_bool("is_important")
-            body = req_str("body")
-            note = g_supply_note["notes"][index]
-            note["category"] = category
-            note["is_important"] = is_important
-            note["body"] = body
-            g_supply.note = str(g_supply_note)
-            g.sess.commit()
-            return chellow_redirect(f"/supplies/{g_supply_id}/notes", 303)
+        g_supply_note = loads(g_supply.note)
+        body = req_str("body")
+        note = g_supply_note["notes"][index]
+        note["body"] = body
+        g_supply.note = dumps(g_supply_note)
+        g.sess.commit()
+        return hx_redirect(f"/supplies/{g_supply_id}/notes/{index}")
     except BadRequest as e:
         flash(e.description)
-        g_supply_note = eval(g_supply.note)
+        g_supply_note = loads(g_supply.note)
         note = g_supply_note["notes"][index]
         note["index"] = index
         return render_template("supply_note_edit.html", g_supply=g_supply, note=note)
@@ -768,7 +809,11 @@ def supply_note_edit_post(g_supply_id, index):
 @gas.route("/eras/<int:g_era_id>/edit")
 def era_edit_get(g_era_id):
     g_era = GEra.get_by_id(g.sess, g_era_id)
-    supplier_g_contracts = g.sess.query(GContract).order_by(GContract.name)
+    supplier_g_contracts = g.sess.scalars(
+        select(GContract)
+        .where(GContract.is_industry == false())
+        .order_by(GContract.name)
+    )
     site_g_eras = (
         g.sess.query(SiteGEra)
         .join(Site)
@@ -837,6 +882,8 @@ def era_edit_post(g_era_id):
             g_reading_frequency = GReadingFrequency.get_by_id(
                 g.sess, g_reading_frequency_id
             )
+            aq = req_decimal("aq")
+            soq = req_decimal("soq")
             g_era.g_supply.update_g_era(
                 g.sess,
                 g_era,
@@ -848,6 +895,8 @@ def era_edit_post(g_era_id):
                 g_contract,
                 account,
                 g_reading_frequency,
+                aq,
+                soq,
             )
             g.sess.commit()
             return chellow_redirect(f"/supplies/{g_era.g_supply.id}", 303)
@@ -1209,7 +1258,7 @@ def industry_contract_edit_patch(g_contract_id):
             g_contract.update(name, charge_script, properties)
 
         g.sess.commit()
-        return chellow_redirect(f"/industry_contracts/{g_contract.id}", 204)
+        return hx_redirect(f"/industry_contracts/{g_contract.id}", 303)
     except BadRequest as e:
         flash(e.description)
         g.sess.rollback()
@@ -1407,47 +1456,88 @@ def exit_zone_get(g_exit_zone_id):
 @gas.route("/batches/<int:g_batch_id>/csv")
 def batch_csv_get(g_batch_id):
     g_batch = GBatch.get_by_id(g.sess, g_batch_id)
+    rows = []
+    max_reads = 0
+    for g_bill in g.sess.scalars(
+        select(GBill)
+        .where(GBill.g_batch == g_batch)
+        .order_by(GBill.reference, GBill.start_date)
+        .options(
+            joinedload(GBill.bill_type),
+            joinedload(GBill.g_reads).joinedload(GRegisterRead.g_unit),
+            joinedload(GBill.g_reads).joinedload(GRegisterRead.pres_type),
+            joinedload(GBill.g_reads).joinedload(GRegisterRead.prev_type),
+        )
+    ).unique():
+        row = [
+            g_batch.g_contract.name,
+            g_batch.reference,
+            g_bill.reference,
+            g_bill.account,
+            g_bill.issue_date,
+            g_bill.start_date,
+            g_bill.finish_date,
+            g_bill.kwh,
+            g_bill.net,
+            g_bill.vat,
+            g_bill.gross,
+            g_bill.bill_type.code,
+            g_bill.breakdown,
+        ]
+        g_reads = g_bill.g_reads
+        max_reads = max(max_reads, len(g_reads))
+        for g_read in g_reads:
+            row.extend(
+                [
+                    g_read.msn,
+                    g_read.g_unit.code,
+                    g_read.correction_factor,
+                    g_read.calorific_value,
+                    g_read.prev_date,
+                    g_read.prev_value,
+                    g_read.prev_type.code,
+                    g_read.pres_date,
+                    g_read.pres_value,
+                    g_read.pres_type.code,
+                ]
+            )
+        rows.append(row)
+
     si = StringIO()
     cw = csv.writer(si)
-    cw.writerow(
-        [
-            "Contract",
-            "Batch Reference",
-            "Bill Reference",
-            "Account",
-            "Issued",
-            "From",
-            "To",
-            "kWh",
-            "Net",
-            "VAT",
-            "Gross",
-            "Type",
-        ]
-    )
-    for g_bill in (
-        g.sess.query(GBill)
-        .filter(GBill.g_batch == g_batch)
-        .order_by(GBill.reference, GBill.start_date)
-        .options(joinedload(GBill.bill_type))
-    ):
-        cw.writerow(
-            [
-                g_batch.g_contract.name,
-                g_batch.reference,
-                g_bill.reference,
-                g_bill.account,
-                hh_format(g_bill.issue_date),
-                hh_format(g_bill.start_date),
-                hh_format(g_bill.finish_date),
-                str(g_bill.kwh),
-                str(g_bill.net),
-                str(g_bill.vat),
-                str(g_bill.gross),
-                g_bill.bill_type.code,
-            ]
-        )
-
+    read_titles = [
+        "msn",
+        "unit",
+        "correction_factor",
+        "calorific_value",
+        "prev_date",
+        "prev_value",
+        "prev_type",
+        "pres_date",
+        "pres_value",
+        "pres_type",
+    ]
+    titles = [
+        "Contract",
+        "Batch Reference",
+        "Bill Reference",
+        "Account",
+        "Issued",
+        "From",
+        "To",
+        "kWh",
+        "Net",
+        "VAT",
+        "Gross",
+        "Type",
+        "breakdown",
+    ]
+    for i in range(max_reads):
+        for rt in read_titles:
+            titles.append(f"{i}_{rt}")
+    cw.writerow(titles)
+    for row in rows:
+        cw.writerow(csv_make_val(v) for v in row)
     disp = f'attachment; filename="g_batch_{g_batch.id}.csv"'
     output = make_response(si.getvalue())
     output.headers["Content-Disposition"] = disp
@@ -1457,12 +1547,38 @@ def batch_csv_get(g_batch_id):
 
 @gas.route("/supplier_contracts")
 def supplier_contracts_get():
-    contracts = g.sess.execute(
+    GRateScriptAliasFinish = aliased(GRateScript)
+
+    current_contracts = g.sess.scalars(
         select(GContract)
-        .where(GContract.is_industry == false())
+        .join(
+            GRateScriptAliasFinish,
+            GContract.finish_g_rate_script_id == GRateScriptAliasFinish.id,
+        )
+        .where(
+            GContract.is_industry == false(),
+            GRateScriptAliasFinish.finish_date == null(),
+        )
         .order_by(GContract.name)
-    ).scalars()
-    return render_template("supplier_contracts.html", contracts=contracts)
+    )
+    ended_contracts = g.sess.scalars(
+        select(GContract)
+        .join(
+            GRateScriptAliasFinish,
+            GContract.finish_g_rate_script_id == GRateScriptAliasFinish.id,
+        )
+        .where(
+            GContract.is_industry == false(),
+            GRateScriptAliasFinish.finish_date != null(),
+        )
+        .order_by(GContract.name)
+    )
+
+    return render_template(
+        "supplier_contracts.html",
+        current_contracts=current_contracts,
+        ended_contracts=ended_contracts,
+    )
 
 
 @gas.route("/supplier_contracts/<int:contract_id>")
@@ -1530,17 +1646,27 @@ def supplier_contract_edit_get(g_contract_id):
 def supplier_contract_edit_post(g_contract_id):
     try:
         g_contract = GContract.get_by_id(g.sess, g_contract_id)
-        if "delete" in request.values:
-            g_contract.delete(g.sess)
-            g.sess.commit()
-            return chellow_redirect("/supplier_contracts", 303)
-        else:
-            name = req_str("name")
-            charge_script = req_str("charge_script")
-            properties = req_zish("properties")
-            g_contract.update(name, charge_script, properties)
-            g.sess.commit()
-            return chellow_redirect(f"/supplier_contracts/{g_contract.id}", 303)
+        name = req_str("name")
+        charge_script = req_str("charge_script")
+        properties = req_zish("properties")
+        g_contract.update(name, charge_script, properties)
+        g.sess.commit()
+        return chellow_redirect(f"/supplier_contracts/{g_contract.id}", 303)
+    except BadRequest as e:
+        flash(e.description)
+        g.sess.rollback()
+        return make_response(
+            render_template("supplier_contract_edit.html", g_contract=g_contract), 400
+        )
+
+
+@gas.route("/supplier_contracts/<int:g_contract_id>/edit", methods=["DELETE"])
+def supplier_contract_edit_delete(g_contract_id):
+    try:
+        g_contract = GContract.get_by_id(g.sess, g_contract_id)
+        g_contract.delete(g.sess)
+        g.sess.commit()
+        return hx_redirect("/supplier_contracts", 303)
     except BadRequest as e:
         flash(e.description)
         g.sess.rollback()
