@@ -1,18 +1,16 @@
 import csv
-import os
 import threading
 import traceback
-import zipfile
 from io import StringIO
 
-from flask import g, request
+from flask import g, redirect, request
 
 from sqlalchemy import or_, select
 from sqlalchemy.sql.expression import null
 
-import chellow.dloads
+from chellow.dloads import open_file
 from chellow.e.computer import SupplySource, forecast_date
-from chellow.models import Channel, Era, HhDatum, Session, Supply, User
+from chellow.models import Channel, Era, HhDatum, RSession, Supply, User
 from chellow.utils import (
     csv_make_val,
     ct_datetime,
@@ -25,7 +23,6 @@ from chellow.utils import (
     to_ct,
     to_utc,
 )
-from chellow.views import chellow_redirect
 
 
 def _write_row(writer, titles, vals, total):
@@ -48,7 +45,7 @@ def content(
     base_name = ["supplies_hh_data", finish_date_ct.strftime("%Y%m%d%H%M")]
     cache = {}
     try:
-        with Session() as sess:
+        with RSession() as sess:
             supplies = (
                 select(Supply)
                 .join(Era)
@@ -105,13 +102,12 @@ def content(
             fdate = forecast_date()
             user = User.get_by_id(sess, user_id)
 
-            running_name, finished_name = chellow.dloads.make_names(
-                "_".join(base_name) + (".zip" if is_zipped else ".csv"), user
-            )
             if is_zipped:
-                zf = zipfile.ZipFile(running_name, "w", zipfile.ZIP_DEFLATED)
+                zf = open_file(
+                    "_".join(base_name) + ".zip", user, mode="w", is_zip=True
+                )
             else:
-                tf = open(running_name, mode="w", newline="")
+                tf = open_file("_".join(base_name) + ".csv", user, mode="w", newline="")
                 tf.write(titles_csv)
 
             for supply in sess.execute(supplies).scalars():
@@ -226,8 +222,6 @@ def content(
             zf.close()
         else:
             tf.write(msg)
-    finally:
-        os.rename(running_name, finished_name)
 
 
 def do_get(sess):
@@ -271,4 +265,4 @@ def handle_request(mpan_cores=None):
         user.id,
     )
     threading.Thread(target=content, args=args).start()
-    return chellow_redirect("/downloads", 303)
+    return redirect("/downloads", 303)
