@@ -17,8 +17,6 @@ from werkzeug.exceptions import BadRequest
 
 from zish import ZishException, loads
 
-url_root = ""
-
 HH = relativedelta(minutes=30)
 MONTH = relativedelta(months=1)
 YEAR = relativedelta(years=1)
@@ -31,11 +29,15 @@ def req_str(name):
         raise BadRequest(f"The field {name} is required.")
 
 
-def req_bool(name):
+def req_strs(name):
     try:
-        return request.values[name] == "true"
+        return request.values.getlist(name)
     except KeyError:
-        return False
+        raise BadRequest(f"The field {name} is required.")
+
+
+def req_bool(name):
+    return name in request.values
 
 
 def req_int(name):
@@ -45,6 +47,13 @@ def req_int(name):
         raise BadRequest(f"Problem parsing the field {name} as an integer: {e}")
 
 
+def req_ints(name):
+    try:
+        return [int(v) for v in req_strs(name)]
+    except ValueError as e:
+        raise BadRequest(f"Problem parsing the field {name} as an integer: {e}")
+
+
 def req_int_none(name):
     val = req_str(name)
     if val == "":
@@ -69,12 +78,15 @@ def req_zish(name):
 def req_date(prefix, resolution="minute"):
     year = req_int(f"{prefix}_year")
     month = req_int(f"{prefix}_month")
-    day = req_int(f"{prefix}_day")
 
     try:
-        if resolution == "day":
+        if resolution == "month":
+            d = ct_datetime(year, month)
+        elif resolution == "day":
+            day = req_int(f"{prefix}_day")
             d = ct_datetime(year, month, day)
         elif resolution == "minute":
+            day = req_int(f"{prefix}_day")
             hour = req_int(f"{prefix}_hour")
             minute = req_int(f"{prefix}_minute")
             d = ct_datetime(year, month, day, hour, minute)
