@@ -3,6 +3,7 @@ from chellow.models import (
     Comm,
     Contract,
     Cop,
+    DtcMeterType,
     EnergisationStatus,
     GspGroup,
     MarketRole,
@@ -18,6 +19,7 @@ from chellow.models import (
     VoltageLevel,
     insert_comms,
     insert_cops,
+    insert_dtc_meter_types,
     insert_energisation_statuses,
     insert_sources,
     insert_voltage_levels,
@@ -55,8 +57,8 @@ def test_make_site_deltas(mocker):
         {
             "start-date": utc_datetime(2019, 3, 1),
             "used-kwh": 0,
-            "export-net-kwh": 0,
-            "import-net-kwh": 0,
+            "export-grid-kwh": 0,
+            "import-grid-kwh": 0,
             "msp-kwh": 0,
         }
     ]
@@ -72,7 +74,73 @@ def test_make_site_deltas(mocker):
         sess, report_context, site, scenario_hh, forecast_from, supply_id
     )
 
-    assert len(res["supply_deltas"][False]["net"]["site"]) == 0
+    assert len(res["supply_deltas"][False]["grid"]["site"]) == 0
+
+
+def test_make_site_deltas_hh_multi_month(mocker):
+    era_1 = mocker.Mock()
+    era_1.start_date = utc_datetime(2018, 1, 1)
+    era_1.finish_date = None
+    filter_returns = iter([[era_1], []])
+
+    class Sess:
+        def query(self, *args):
+            return self
+
+        def join(self, *args):
+            return self
+
+        def filter(self, *args):
+            return next(filter_returns)
+
+    sess = Sess()
+    report_context = {}
+    site = mocker.Mock()
+    site.code = "1"
+    scenario_hh = {site.code: {"used": "2019-03-01 00:00, 0"}}
+    forecast_from = utc_datetime(2019, 4, 1)
+    supply_id = None
+
+    ss = mocker.patch("chellow.e.scenario.SiteSource", autospec=True)
+    ss_instance = ss.return_value
+    ss_instance.hh_data = [
+        {
+            "start-date": utc_datetime(2019, 3, 1),
+            "used-kwh": 0,
+            "export-grid-kwh": 0,
+            "import-grid-kwh": 0,
+            "msp-kwh": 0,
+        }
+    ]
+
+    se = mocker.patch("chellow.e.scenario.SiteEra", autospec=True)
+    se.site = mocker.Mock()
+
+    sup_s = mocker.patch("chellow.e.scenario.SupplySource", autospec=True)
+    sup_s_instance = sup_s.return_value
+    sup_s_instance.hh_data = {}
+
+    res = make_site_deltas(
+        sess, report_context, site, scenario_hh, forecast_from, supply_id
+    )
+    filter_returns = iter([[era_1], []])
+
+    class Sess:
+        def query(self, *args):
+            return self
+
+        def join(self, *args):
+            return self
+
+        def filter(self, *args):
+            return next(filter_returns)
+
+    sess = Sess()
+    res = make_site_deltas(
+        sess, report_context, site, scenario_hh, forecast_from, supply_id
+    )
+
+    assert len(res["supply_deltas"][False]["grid"]["site"]) == 0
 
 
 def test_make_site_deltas_nhh(mocker):
@@ -131,8 +199,8 @@ def test_make_site_deltas_nhh(mocker):
         {
             "start-date": utc_datetime(2019, 3, 1),
             "used-kwh": 0,
-            "export-net-kwh": 0,
-            "import-net-kwh": 0,
+            "export-grid-kwh": 0,
+            "import-grid-kwh": 0,
             "msp-kwh": 0,
         }
     ]
@@ -149,7 +217,7 @@ def test_make_site_deltas_nhh(mocker):
         sess, report_context, site, scenario_hh, forecast_from, supply_id
     )
 
-    assert res["supply_deltas"][True]["net"]["site"] == {hh_start_date: -10.0}
+    assert res["supply_deltas"][True]["grid"]["site"] == {hh_start_date: -10.0}
 
 
 def test_make_calcs_new_generation(mocker, sess):
@@ -288,10 +356,12 @@ def displaced_virtual_bill(ds):
     )
     MtcLlfc.insert(sess, mtc_participant, llfc_imp, vf, None)
     insert_sources(sess)
-    source = Source.get_by_code(sess, "net")
+    source = Source.get_by_code(sess, "grid")
     insert_energisation_statuses(sess)
     energisation_status = EnergisationStatus.get_by_code(sess, "E")
     gsp_group = GspGroup.insert(sess, "_L", "South Western")
+    insert_dtc_meter_types(sess)
+    dtc_meter_type = DtcMeterType.get_by_code(sess, "H")
     site.insert_e_supply(
         sess,
         source,
@@ -312,7 +382,7 @@ def displaced_virtual_bill(ds):
         comm,
         None,
         energisation_status,
-        {},
+        dtc_meter_type,
         "22 7867 6232 781",
         "510",
         imp_supplier_contract,
@@ -359,4 +429,4 @@ def displaced_virtual_bill(ds):
     )
     assert calcs[1][1] == "displaced"
     assert calcs[2][1] == "CI017_extra_gen_TRUE"
-    assert calcs[3][2] == "CI017_extra_net_export"
+    assert calcs[3][2] == "CI017_extra_grid_export"
