@@ -73,6 +73,8 @@ def test_batch_get(client, sess):
         g_contract,
         "d7gthekrg",
         g_reading_frequency_M,
+        1,
+        1,
     )
 
     g_batch = g_contract.insert_g_batch(sess, "b1", "Jan batch")
@@ -150,6 +152,8 @@ def test_batch_edit_post(sess, client):
         g_contract,
         "d7gthekrg",
         g_reading_frequency_M,
+        1,
+        1,
     )
     g_batch = g_contract.insert_g_batch(sess, "b1", "Jan batch")
     sess.commit()
@@ -180,6 +184,136 @@ def test_batch_edit_delete(sess, client):
     )
 
 
+def test_batch_csv_get(sess, client):
+    vf = to_utc(ct_datetime(2000, 1, 1))
+    site = Site.insert(sess, "22488", "Water Works")
+    g_dn = GDn.insert(sess, "EE", "East of England")
+    g_ldz = g_dn.insert_g_ldz(sess, "EA")
+    g_exit_zone = g_ldz.insert_g_exit_zone(sess, "EA1")
+    insert_g_units(sess)
+    g_unit_M3 = GUnit.get_by_code(sess, "M3")
+    g_contract = GContract.insert(sess, False, "Fusion 2020", "", {}, vf, None, {})
+    insert_g_reading_frequencies(sess)
+    g_reading_frequency_M = GReadingFrequency.get_by_code(sess, "M")
+    mprn = "750278673"
+    g_supply = site.insert_g_supply(
+        sess,
+        mprn,
+        "main",
+        g_exit_zone,
+        utc_datetime(2018, 1, 1),
+        None,
+        "hgeu8rhg",
+        1,
+        g_unit_M3,
+        g_contract,
+        "d7gthekrg",
+        g_reading_frequency_M,
+        1,
+        1,
+    )
+    g_batch = g_contract.insert_g_batch(sess, "b1", "Jan batch")
+    breakdown = {"units_consumed": 771}
+
+    insert_bill_types(sess)
+
+    bill_type_n = BillType.get_by_code(sess, "N")
+
+    g_bill = g_batch.insert_g_bill(
+        sess,
+        g_supply,
+        bill_type_n,
+        "55h883",
+        "dhgh883",
+        utc_datetime(2019, 4, 3),
+        utc_datetime(2020, 1, 1),
+        utc_datetime(2020, 1, 31, 23, 30),
+        Decimal("45"),
+        Decimal("12.40"),
+        Decimal("1.20"),
+        Decimal("14.52"),
+        "",
+        breakdown,
+    )
+    insert_g_read_types(sess)
+
+    g_read_type_A = GReadType.get_by_code(sess, "A")
+
+    g_bill.insert_g_read(
+        sess,
+        "ghu5438gt",
+        g_unit_M3,
+        1,
+        37,
+        Decimal(800),
+        utc_datetime(2020, 1, 1),
+        g_read_type_A,
+        Decimal(900),
+        utc_datetime(2020, 1, 31),
+        g_read_type_A,
+    )
+    sess.commit()
+
+    response = client.get(f"/g/batches/{g_batch.id}/csv")
+    match(response, 200)
+
+    lines = [
+        [
+            "Contract",
+            "Batch Reference",
+            "Bill Reference",
+            "Account",
+            "Issued",
+            "From",
+            "To",
+            "kWh",
+            "Net",
+            "VAT",
+            "Gross",
+            "Type",
+            "breakdown",
+            "0_msn",
+            "0_unit",
+            "0_correction_factor",
+            "0_calorific_value",
+            "0_prev_date",
+            "0_prev_value",
+            "0_prev_type",
+            "0_pres_date",
+            "0_pres_value",
+            "0_pres_type",
+        ],
+        [
+            "Fusion 2020",
+            "b1",
+            "55h883",
+            "dhgh883",
+            "2019-04-03 01:00",
+            "2020-01-01 00:00",
+            "2020-01-31 23:30",
+            "45",
+            "12.40",
+            "1.20",
+            "14.52",
+            "N",
+            '"{\n  ""units_consumed"": 771,\n}"',
+            "ghu5438gt",
+            "M3",
+            "1",
+            "37",
+            "2020-01-01 00:00",
+            "800",
+            "A",
+            "2020-01-31 00:00",
+            "900",
+            "A",
+        ],
+    ]
+    pattern = "\r\n".join(",".join(line) for line in lines) + "\r\n"
+    actual = response.get_data(as_text=True)
+    assert pattern == actual
+
+
 def test_bill_get(client, sess):
     site = Site.insert(sess, "22488", "Water Works")
 
@@ -214,6 +348,8 @@ def test_bill_get(client, sess):
         g_contract,
         "d7gthekrg",
         g_reading_frequency_M,
+        1,
+        1,
     )
 
     g_batch = g_contract.insert_g_batch(sess, "b1", "Jan batch")
@@ -274,6 +410,8 @@ def test_bill_add_post(sess, client):
         g_contract,
         "d7gthekrg",
         g_reading_frequency_M,
+        1,
+        1,
     )
     g_batch = g_contract.insert_g_batch(sess, "b1", "Jan batch")
     insert_bill_types(sess)
@@ -336,6 +474,8 @@ def test_bill_edit_post(sess, client):
         g_contract,
         "d7gthekrg",
         g_reading_frequency_M,
+        1,
+        1,
     )
     g_batch = g_contract.insert_g_batch(sess, "b1", "Jan batch")
 
@@ -500,6 +640,8 @@ def test_bill_imports_post_full(mocker, app, client, sess):
         g_contract,
         "d7gthekrg",
         g_reading_frequency_M,
+        1,
+        1,
     )
 
     insert_g_read_types(sess)
@@ -536,6 +678,7 @@ def test_bill_imports_post_full(mocker, app, client, sess):
         response,
         200,
         r"All the bills have been successfully loaded and attached to the batch.",
+        r"<td>M3</td>",
     )
 
     sess.rollback()
@@ -649,7 +792,7 @@ def test_industry_contract_edit_patch(client, sess):
         "state": '{"hinderance": "sloth"}',
     }
     response = client.patch(f"/g/industry_contracts/{contract.id}/edit", data=data)
-    match(response, 204)
+    match(response, 303)
     response = client.get(f"/g/industry_contracts/{contract.id}")
     match(response, 200, "&#34;hinderance&#34;: &#34;sloth&#34;")
 
@@ -677,6 +820,22 @@ def test_supplier_contracts_empty(client, sess):
     match(response, 200)
 
 
+def test_supplier_contract_edit_delete(client, sess):
+    contract = GContract.insert_supplier(
+        sess,
+        "Fusion 2020",
+        "",
+        {},
+        utc_datetime(2019, 1, 1),
+        None,
+        {},
+    )
+    sess.commit()
+
+    response = client.delete(f"/g/supplier_contracts/{contract.id}/edit")
+    assert response.headers["HX-Redirect"].endswith("/g/supplier_contracts")
+
+
 def test_supplies(client, sess):
     response = client.get("/g/supplies")
 
@@ -712,6 +871,8 @@ def test_supply_get(client, sess):
         g_contract,
         "d7gthekrg",
         g_reading_frequency_M,
+        1,
+        1,
     )
 
     sess.commit()
@@ -721,6 +882,69 @@ def test_supply_get(client, sess):
     match(response, 200)
 
 
+def test_supply_get_bill_after_end(client, sess):
+    site = Site.insert(sess, "22488", "Water Works")
+
+    g_dn = GDn.insert(sess, "EE", "East of England")
+    g_ldz = g_dn.insert_g_ldz(sess, "EA")
+    g_exit_zone = g_ldz.insert_g_exit_zone(sess, "EA1")
+    insert_g_units(sess)
+    g_unit_M3 = GUnit.get_by_code(sess, "M3")
+
+    g_contract = GContract.insert_supplier(
+        sess, "Fusion 2020", "", {}, utc_datetime(2000, 1, 1), None, {}
+    )
+
+    insert_g_reading_frequencies(sess)
+    g_reading_frequency_M = GReadingFrequency.get_by_code(sess, "M")
+
+    g_supply = site.insert_g_supply(
+        sess,
+        "7y94u5",
+        "main",
+        g_exit_zone,
+        to_utc(ct_datetime(2018, 1, 1)),
+        to_utc(ct_datetime(2018, 1, 10)),
+        "hgeu8rhg",
+        1,
+        g_unit_M3,
+        g_contract,
+        "d7gthekrg",
+        g_reading_frequency_M,
+        1,
+        1,
+    )
+    g_batch = g_contract.insert_g_batch(sess, "b1", "Jan batch")
+
+    insert_bill_types(sess)
+
+    bill_type_n = BillType.get_by_code(sess, "N")
+
+    g_batch.insert_g_bill(
+        sess,
+        g_supply,
+        bill_type_n,
+        "55h883",
+        "dhgh883",
+        utc_datetime(2019, 4, 3),
+        utc_datetime(2020, 1, 1),
+        utc_datetime(2020, 1, 31, 23, 30),
+        Decimal("45"),
+        Decimal("12.40"),
+        Decimal("1.20"),
+        Decimal("14.52"),
+        "",
+        {},
+    )
+
+    sess.commit()
+
+    response = client.get(f"/g/supplies/{g_supply.id}")
+    patterns = [r'<td rowspan="1">\s*' r'<a href="/g/bills/1">view</a>\s*' r"</td>"]
+
+    match(response, 200, *patterns)
+
+
 def test_supply_note_add_get(client, sess):
     site = Site.insert(sess, "22488", "Water Works")
 
@@ -754,6 +978,8 @@ def test_supply_note_add_get(client, sess):
         g_contract,
         "d7gthekrg",
         g_reading_frequency_M,
+        1,
+        1,
     )
 
     sess.commit()
@@ -795,6 +1021,8 @@ def test_supply_notes_get(client, sess):
         g_contract,
         "d7gthekrg",
         g_reading_frequency_M,
+        1,
+        1,
     )
     sess.commit()
 
@@ -832,6 +1060,8 @@ def test_supply_edit_post(client, sess):
         g_contract,
         "d7gthekrg",
         g_reading_frequency_M,
+        1,
+        1,
     )
 
     sess.commit()
@@ -846,7 +1076,7 @@ def test_supply_edit_post(client, sess):
     }
 
     response = client.post(f"/g/supplies/{g_supply.id}/edit", data=data)
-    assert response.headers["Location"] == f"http://localhost/g/supplies/{g_supply.id}"
+    assert response.headers["Location"] == f"/g/supplies/{g_supply.id}"
 
     match(response, 303)
 
@@ -885,6 +1115,8 @@ def test_read_edit_post_delete(sess, client):
         g_contract,
         "d7gthekrg",
         g_reading_frequency_M,
+        1,
+        1,
     )
 
     g_batch = g_contract.insert_g_batch(sess, "b1", "Jan batch")
@@ -939,6 +1171,44 @@ def test_read_edit_post_delete(sess, client):
     match(response, 303, f"/g/bills/{g_bill.id}")
 
 
+def test_supply_edit_get(client, sess):
+    vf = to_utc(ct_datetime(2000, 1, 1))
+    site = Site.insert(sess, "22488", "Water Works")
+
+    g_dn = GDn.insert(sess, "EE", "East of England")
+    g_ldz = g_dn.insert_g_ldz(sess, "EA")
+    g_exit_zone = g_ldz.insert_g_exit_zone(sess, "EA1")
+    insert_g_units(sess)
+    g_unit_M3 = GUnit.get_by_code(sess, "M3")
+
+    g_contract = GContract.insert_supplier(sess, "Fusion 2020", "", {}, vf, None, {})
+
+    insert_g_reading_frequencies(sess)
+    g_reading_frequency_M = GReadingFrequency.get_by_code(sess, "M")
+
+    g_supply = site.insert_g_supply(
+        sess,
+        "7y94u5",
+        "main",
+        g_exit_zone,
+        utc_datetime(2018, 1, 1),
+        None,
+        "hgeu8rhg",
+        1,
+        g_unit_M3,
+        g_contract,
+        "d7gthekrg",
+        g_reading_frequency_M,
+        1,
+        1,
+    )
+
+    sess.commit()
+
+    response = client.get(f"/g/supplies/{g_supply.id}/edit")
+    match(response, 200, r'<option value="1" selected>EA1</option>')
+
+
 def test_supplier_rate_script_get(sess, client):
     g_contract = GContract.insert(
         sess,
@@ -1022,6 +1292,8 @@ def test_read_add_get(sess, client):
         g_contract,
         "d7gthekrg",
         g_reading_frequency_M,
+        1,
+        1,
     )
 
     g_batch = g_contract.insert_g_batch(sess, "b1", "Jan batch")
@@ -1080,6 +1352,8 @@ def test_read_add_post(sess, client):
         g_contract,
         "d7gthekrg",
         g_reading_frequency_M,
+        1,
+        1,
     )
 
     g_batch = g_contract.insert_g_batch(sess, "b1", "Jan batch")
@@ -1135,6 +1409,114 @@ def test_read_add_post(sess, client):
     match(response, 303)
 
 
+def test_era_edit_get(sess, client):
+    vf = to_utc(ct_datetime(2000, 1, 1))
+    site = Site.insert(sess, "22488", "Water Works")
+    g_dn = GDn.insert(sess, "EE", "East of England")
+    g_ldz = g_dn.insert_g_ldz(sess, "EA")
+    g_exit_zone = g_ldz.insert_g_exit_zone(sess, "EA1")
+    insert_g_units(sess)
+    g_unit_M3 = GUnit.get_by_code(sess, "M3")
+    insert_g_reading_frequencies(sess)
+    g_reading_frequency_M = GReadingFrequency.get_by_code(sess, "M")
+    GContract.insert(sess, True, "Unidentified gas", "", {}, vf, None, {})
+    g_contract = GContract.insert(sess, False, "Fusion 2020", "", {}, vf, None, {})
+
+    g_supply = site.insert_g_supply(
+        sess,
+        "7y94u5",
+        "main",
+        g_exit_zone,
+        utc_datetime(2018, 1, 1),
+        None,
+        "hgeu8rhg",
+        1,
+        g_unit_M3,
+        g_contract,
+        "d7gthekrg",
+        g_reading_frequency_M,
+        1,
+        1,
+    )
+
+    g_era = g_supply.g_eras[0]
+
+    sess.commit()
+
+    response = client.get(f"/g/eras/{g_era.id}/edit")
+
+    match(
+        response,
+        200,
+        r'<select name="g_contract_id">\s*'
+        r'<option value="2" selected>Fusion 2020</option>\s*'
+        r"</select>",
+    )
+
+
+def test_era_post(sess, client):
+    site = Site.insert(sess, "22488", "Water Works")
+
+    g_dn = GDn.insert(sess, "EE", "East of England")
+
+    g_ldz = g_dn.insert_g_ldz(sess, "EA")
+
+    g_exit_zone = g_ldz.insert_g_exit_zone(sess, "EA1")
+
+    insert_g_units(sess)
+
+    g_unit_M3 = GUnit.get_by_code(sess, "M3")
+
+    insert_g_reading_frequencies(sess)
+
+    g_reading_frequency_M = GReadingFrequency.get_by_code(sess, "M")
+
+    g_contract = GContract.insert(
+        sess, False, "Fusion 2020", "", {}, utc_datetime(2000, 1, 1), None, {}
+    )
+
+    g_supply = site.insert_g_supply(
+        sess,
+        "7y94u5",
+        "main",
+        g_exit_zone,
+        utc_datetime(2018, 1, 1),
+        None,
+        "hgeu8rhg",
+        1,
+        g_unit_M3,
+        g_contract,
+        "d7gthekrg",
+        g_reading_frequency_M,
+        1,
+        1,
+    )
+
+    g_era = g_supply.g_eras[0]
+
+    sess.commit()
+
+    data = {
+        "start_year": "2018",
+        "start_month": "01",
+        "start_day": "01",
+        "start_hour": "0",
+        "start_minute": "0",
+        "msn": "9874h6l",
+        "correction_factor": "1",
+        "g_contract_id": g_contract.id,
+        "account": "acc 8",
+        "g_unit_id": g_unit_M3.id,
+        "g_reading_frequency_id": g_reading_frequency_M.id,
+        "aq": 1,
+        "soq": 1,
+    }
+
+    response = client.post(f"/g/eras/{g_era.id}/edit", data=data)
+
+    match(response, 303, r"/g/supplies/1")
+
+
 def test_era_post_delete(sess, client):
     site = Site.insert(sess, "22488", "Water Works")
 
@@ -1169,6 +1551,8 @@ def test_era_post_delete(sess, client):
         g_contract,
         "d7gthekrg",
         g_reading_frequency_M,
+        1,
+        1,
     )
 
     g_era = g_supply.insert_g_era_at(sess, utc_datetime(2018, 3, 1))
