@@ -7,6 +7,7 @@ from chellow.models import (
     Comm,
     Contract,
     Cop,
+    DtcMeterType,
     EnergisationStatus,
     GspGroup,
     MarketRole,
@@ -24,6 +25,7 @@ from chellow.models import (
     VoltageLevel,
     insert_comms,
     insert_cops,
+    insert_dtc_meter_types,
     insert_energisation_statuses,
     insert_sources,
     insert_voltage_levels,
@@ -32,8 +34,8 @@ from chellow.reports.report_169 import content
 from chellow.utils import ct_datetime, to_utc, utc_datetime
 
 
-def test_handle_request(mocker, client, sess):
-    user = User.get_by_email_address(sess, "admin@example.com")
+def test_handle_request(mocker, client, rsess):
+    user = User.get_by_email_address(rsess, "admin@example.com")
     MockThread = mocker.patch("chellow.reports.report_169.threading.Thread")
 
     channel_type = "used"
@@ -65,16 +67,14 @@ def test_handle_request(mocker, client, sess):
     MockThread.assert_called_with(target=content, args=expected_args)
 
 
-def test_content(mocker, client, sess):
-    user = User.get_by_email_address(sess, "admin@example.com")
+def test_content(mocker, client, rsess):
+    user = User.get_by_email_address(rsess, "admin@example.com")
 
     mock_file = StringIO()
     mock_file.close = mocker.Mock()
-    mocker.patch("chellow.reports.report_169.open", return_value=mock_file)
-    mock_make_names = mocker.patch(
-        "chellow.reports.report_169.chellow.dloads.make_names", return_value=("a", "b")
+    mock_open_file = mocker.patch(
+        "chellow.reports.report_169.open_file", return_value=mock_file
     )
-    mocker.patch("chellow.reports.report_169.os.rename")
     start_date = to_utc(ct_datetime(2020, 6, 1))
     finish_date = to_utc(ct_datetime(2020, 6, 1, 23, 30))
     imp_related = True
@@ -93,7 +93,7 @@ def test_content(mocker, client, sess):
         mpan_cores,
         user_id,
     )
-    call_args = mock_make_names.call_args
+    call_args = mock_open_file.call_args
     arg_name, arg_user = call_args[0]
     assert arg_name == "supplies_hh_data_202006012330.csv"
     assert arg_user.id == user.id
@@ -216,10 +216,12 @@ def test_content_zip(mocker, sess):
     )
     MtcLlfc.insert(sess, mtc_participant, llfc, vf, None)
     insert_sources(sess)
-    source = Source.get_by_code(sess, "net")
+    source = Source.get_by_code(sess, "grid")
     gsp_group = GspGroup.insert(sess, "_L", "South Western")
     insert_energisation_statuses(sess)
     energisation_status = EnergisationStatus.get_by_code(sess, "E")
+    insert_dtc_meter_types(sess)
+    dtc_meter_type = DtcMeterType.get_by_code(sess, "H")
     site.insert_e_supply(
         sess,
         source,
@@ -240,7 +242,7 @@ def test_content_zip(mocker, sess):
         comm,
         None,
         energisation_status,
-        {},
+        dtc_meter_type,
         "22 7867 6232 781",
         "510",
         imp_supplier_contract,
@@ -255,16 +257,13 @@ def test_content_zip(mocker, sess):
 
     editor = UserRole.insert(sess, "editor")
     user = User.insert(sess, "admin@example.com", "xxx", editor, None)
+    user_id = user.id
 
     sess.commit()
 
     f = BytesIO()
     zf = ZipFile(f, "w", ZIP_DEFLATED)
-    mocker.patch("chellow.reports.report_169.zipfile.ZipFile", return_value=zf)
-    mocker.patch(
-        "chellow.reports.report_169.chellow.dloads.make_names", return_value=("a", "b")
-    )
-    mocker.patch("chellow.reports.report_169.os.rename")
+    mocker.patch("chellow.reports.report_169.open_file", return_value=zf)
     start_date = to_utc(ct_datetime(2020, 6, 1))
     finish_date = to_utc(ct_datetime(2020, 6, 1, 23, 30))
     imp_related = True
@@ -272,7 +271,6 @@ def test_content_zip(mocker, sess):
     is_zipped = True
     supply_id = None
     mpan_cores = None
-    user_id = user.id
     content(
         start_date,
         finish_date,
