@@ -4,6 +4,7 @@ from chellow.models import (
     Comm,
     Contract,
     Cop,
+    DtcMeterType,
     EnergisationStatus,
     GspGroup,
     MarketRole,
@@ -21,6 +22,7 @@ from chellow.models import (
     VoltageLevel,
     insert_comms,
     insert_cops,
+    insert_dtc_meter_types,
     insert_energisation_statuses,
     insert_sources,
     insert_voltage_levels,
@@ -32,12 +34,7 @@ from chellow.utils import ct_datetime, to_utc, utc_datetime
 def test_content_no_data(mocker, sess):
     mock_file = StringIO()
     mock_file.close = mocker.Mock()
-    mocker.patch("chellow.reports.report_187.open", return_value=mock_file)
-    mocker.patch(
-        "chellow.reports.report_187.chellow.dloads.make_names",
-        return_value=("a", "b"),
-    )
-    mocker.patch("chellow.reports.report_187.os.rename")
+    mocker.patch("chellow.reports.report_187.open_file", return_value=mock_file)
     start_date = to_utc(ct_datetime(2023, 7, 5))
     finish_date = to_utc(ct_datetime(2023, 7, 18, 22, 30))
     supply_id = None
@@ -125,10 +122,12 @@ def test_content(mocker, sess):
     )
     MtcLlfc.insert(sess, mtc_participant, llfc, vf, None)
     insert_sources(sess)
-    source = Source.get_by_code(sess, "net")
+    source = Source.get_by_code(sess, "grid")
     gsp_group = GspGroup.insert(sess, "_L", "South Western")
     insert_energisation_statuses(sess)
     energisation_status = EnergisationStatus.get_by_code(sess, "E")
+    insert_dtc_meter_types(sess)
+    dtc_meter_type = DtcMeterType.get_by_code(sess, "H")
     site.insert_e_supply(
         sess,
         source,
@@ -149,7 +148,7 @@ def test_content(mocker, sess):
         comm,
         None,
         energisation_status,
-        {},
+        dtc_meter_type,
         "22 7867 6232 781",
         "510",
         imp_supplier_contract,
@@ -167,12 +166,7 @@ def test_content(mocker, sess):
 
     mock_file = StringIO()
     mock_file.close = mocker.Mock()
-    mocker.patch("chellow.reports.report_187.open", return_value=mock_file)
-    mocker.patch(
-        "chellow.reports.report_187.chellow.dloads.make_names",
-        return_value=("a", "b"),
-    )
-    mocker.patch("chellow.reports.report_187.os.rename")
+    mocker.patch("chellow.reports.report_187.open_file", return_value=mock_file)
     start_date = to_utc(ct_datetime(2023, 7, 18, 22, 30))
     finish_date = to_utc(ct_datetime(2023, 7, 18, 22, 30))
     supply_id = None
