@@ -4,6 +4,7 @@ from chellow.models import (
     Comm,
     Contract,
     Cop,
+    DtcMeterType,
     EnergisationStatus,
     GspGroup,
     MarketRole,
@@ -21,6 +22,7 @@ from chellow.models import (
     VoltageLevel,
     insert_comms,
     insert_cops,
+    insert_dtc_meter_types,
     insert_energisation_statuses,
     insert_sources,
     insert_voltage_levels,
@@ -74,10 +76,12 @@ def test_content(mocker, sess):
     llfc = dno.insert_llfc(sess, "510", "HV", voltage_level, False, True, vf, None)
     MtcLlfc.insert(sess, mtc_participant, llfc, vf, None)
     insert_sources(sess)
-    source = Source.get_by_code(sess, "net")
+    source = Source.get_by_code(sess, "grid")
     insert_energisation_statuses(sess)
     energisation_status = EnergisationStatus.get_by_code(sess, "E")
     gsp_group = GspGroup.insert(sess, "_L", "South Western")
+    insert_dtc_meter_types(sess)
+    dtc_meter_type = DtcMeterType.get_by_code(sess, "H")
     supply = site.insert_e_supply(
         sess,
         source,
@@ -98,7 +102,7 @@ def test_content(mocker, sess):
         comm,
         None,
         energisation_status,
-        {},
+        dtc_meter_type,
         "22 7867 6232 781",
         "510",
         imp_supplier_contract,
@@ -126,11 +130,7 @@ def test_content(mocker, sess):
     sess.commit()
     f = StringIO()
     f.close = mocker.Mock()
-    mocker.patch("chellow.reports.report_29.open", return_value=f)
-    mocker.patch("chellow.reports.report_29.os.rename")
-    mocker.patch(
-        "chellow.reports.report_29.chellow.dloads.make_names", return_value=("a", "b")
-    )
+    mocker.patch("chellow.reports.report_29.open_file", return_value=f)
     start_date = to_utc(ct_datetime(2010, 1, 1))
     finish_date = to_utc(ct_datetime(2010, 1, 31, 23, 30))
     site_id = site.id
