@@ -1,9 +1,13 @@
+import csv
 from io import StringIO
 
+from utils import match_tables
+
 from chellow.models import (
     Comm,
     Contract,
     Cop,
+    DtcMeterType,
     EnergisationStatus,
     GspGroup,
     MarketRole,
@@ -11,24 +15,21 @@ from chellow.models import (
     MeterType,
     Mtc,
     MtcLlfc,
-    MtcLlfcSsc,
-    MtcLlfcSscPc,
     MtcParticipant,
-    MtcSsc,
     Participant,
     Pc,
     ReportRun,
     Site,
     Source,
-    Ssc,
     VoltageLevel,
     insert_comms,
     insert_cops,
+    insert_dtc_meter_types,
     insert_energisation_statuses,
     insert_sources,
     insert_voltage_levels,
 )
-from chellow.reports.report_ecoes_comparison import _meter_type, _process
+from chellow.reports.report_ecoes_comparison import _process
 from chellow.utils import ct_datetime, to_utc, utc_datetime
 
 
@@ -108,10 +109,12 @@ def test_process(mocker, sess):
     )
     MtcLlfc.insert(sess, mtc_participant, llfc, valid_from, None)
     insert_sources(sess)
-    source = Source.get_by_code(sess, "net")
+    source = Source.get_by_code(sess, "grid")
     insert_energisation_statuses(sess)
     energisation_status = EnergisationStatus.get_by_code(sess, "E")
     gsp_group = GspGroup.insert(sess, "_L", "South Western")
+    insert_dtc_meter_types(sess)
+    dtc_meter_type = DtcMeterType.get_by_code(sess, "H")
     site.insert_e_supply(
         sess,
         source,
@@ -132,7 +135,7 @@ def test_process(mocker, sess):
         comm,
         None,
         energisation_status,
-        {},
+        dtc_meter_type,
         "22 7867 6232 781",
         "510",
         imp_supplier_contract,
@@ -165,7 +168,7 @@ def test_process(mocker, sess):
         comm,
         None,
         energisation_status,
-        {},
+        dtc_meter_type,
         "22 7868 6232 789",
         "510",
         imp_supplier_contract,
@@ -282,25 +285,30 @@ def test_process(mocker, sess):
             "ecoes_pc",
             "chellow_pc",
             "ecoes_mtc",
+            "ecoes_mtc_date",
             "chellow_mtc",
             "ecoes_llfc",
+            "ecoes_llfc_from",
             "chellow_llfc",
             "ecoes_ssc",
             "chellow_ssc",
             "ecoes_es",
             "chellow_es",
             "ecoes_supplier",
+            "ecoes_supplier_registration_from",
             "chellow_supplier",
             "chellow_supplier_contract_name",
             "ecoes_dc",
             "chellow_dc",
             "ecoes_mop",
+            "ecoes_mop_appoint_date",
             "chellow_mop",
             "ecoes_gsp_group",
+            "ecoes_gsp_effective_from",
             "chellow_gsp_group",
             "ecoes_msn",
-            "chellow_msn",
             "ecoes_msn_install_date",
+            "chellow_msn",
             "ecoes_meter_type",
             "chellow_meter_type",
             "ignored",
@@ -312,25 +320,30 @@ def test_process(mocker, sess):
             "pc",
             "00",
             "mtc",
+            "2022-10-12 00:00",
             "845",
             "llfc",
+            "2022-10-12 00:00",
             "510",
             "",
             "",
             "energisation-status",
             "E",
             "supplier",
+            "2022-10-12 00:00",
             "CALB",
             "Fusion Supplier 2000",
             "dc",
             "CALB",
             "mop",
+            "2022-10-12 00:00",
             "CALB",
             "gsp-group",
+            "2022-10-12 00:00",
             "_L",
             "msn",
-            "hgjeyhuw",
             "2022-10-12 00:00",
+            "hgjeyhuw",
             "meter-type",
             "H",
             "False",
@@ -346,25 +359,30 @@ def test_process(mocker, sess):
             "pc",
             "00",
             "mtc",
+            "2022-10-12 00:00",
             "845",
             "llfc",
+            "2022-10-12 00:00",
             "510",
             "",
             "",
             "energisation-status",
             "E",
             "supplier",
+            "2022-10-12 00:00",
             "CALB",
             "Fusion Supplier 2000",
             "dc",
             "CALB",
             "mop",
+            "2022-10-12 00:00",
             "CALB",
             "gsp-group",
+            "2022-10-12 00:00",
             "_L",
             "msn",
-            "hgjeyhuw",
             "2022-10-12 00:00",
+            "hgjeyhuw",
             "meter-type",
             "H",
             "False",
@@ -375,60 +393,43 @@ def test_process(mocker, sess):
             "match.",
         ],
     ]
-    expected = "\n".join(",".join(line) for line in expected) + "\n"
-    actual = f.getvalue()
-    assert actual == expected
+    f.seek(0)
+    actual = [line for line in csv.reader(f)]
+    match_tables(expected, actual)
 
 
 def test_process_in_chellow_not_ecoes(mocker, sess):
-    valid_from = to_utc(ct_datetime(2000, 1, 1))
+    vf = to_utc(ct_datetime(2000, 1, 1))
     site = Site.insert(sess, "CI017", "Water Works")
 
     market_role_Z = MarketRole.insert(sess, "Z", "Non-core")
     participant = Participant.insert(sess, "CALB", "AK Industries")
-    participant.insert_party(sess, market_role_Z, "None core", valid_from, None, None)
-    bank_holiday_rate_script = {"bank_holidays": []}
-    Contract.insert_non_core(
-        sess,
-        "bank_holidays",
-        "",
-        {},
-        utc_datetime(2000, 1, 1),
-        None,
-        bank_holiday_rate_script,
-    )
+    participant.insert_party(sess, market_role_Z, "None core", vf, None, None)
     market_role_X = MarketRole.insert(sess, "X", "Supplier")
     market_role_M = MarketRole.insert(sess, "M", "Mop")
     market_role_C = MarketRole.insert(sess, "C", "HH Dc")
     market_role_R = MarketRole.insert(sess, "R", "Distributor")
-    participant.insert_party(sess, market_role_M, "Fusion Mop", valid_from, None, None)
-    participant.insert_party(sess, market_role_X, "Fusion Ltc", valid_from, None, None)
-    participant.insert_party(sess, market_role_C, "Fusion DC", valid_from, None, None)
+    participant.insert_party(sess, market_role_M, "Fusion Mop", vf, None, None)
+    participant.insert_party(sess, market_role_X, "Fusion Ltc", vf, None, None)
+    participant.insert_party(sess, market_role_C, "Fusion DC", vf, None, None)
     mop_contract = Contract.insert_mop(
-        sess, "Fusion", participant, "", {}, utc_datetime(2000, 1, 1), None, {}
+        sess, "Fusion", participant, "", {}, vf, None, {}
     )
     dc_contract = Contract.insert_dc(
-        sess, "Fusion DC 2000", participant, "", {}, utc_datetime(2000, 1, 1), None, {}
+        sess, "Fusion DC 2000", participant, "", {}, vf, None, {}
     )
-    pc = Pc.insert(sess, "00", "hh", utc_datetime(2000, 1, 1), None)
+    pc = Pc.insert(sess, "00", "hh", vf, None)
     insert_cops(sess)
     cop = Cop.get_by_code(sess, "5")
     insert_comms(sess)
     comm = Comm.get_by_code(sess, "GSM")
     imp_supplier_contract = Contract.insert_supplier(
-        sess,
-        "Fusion Supplier 2000",
-        participant,
-        "",
-        {},
-        utc_datetime(2000, 1, 1),
-        None,
-        {},
+        sess, "Fusion Supplier 2000", participant, "", {}, vf, None, {}
     )
-    dno = participant.insert_party(sess, market_role_R, "WPD", valid_from, None, "22")
-    meter_type = MeterType.insert(sess, "C5", "COP 1-5", utc_datetime(2000, 1, 1), None)
-    meter_payment_type = MeterPaymentType.insert(sess, "CR", "Credit", valid_from, None)
-    mtc = Mtc.insert(sess, "845", False, True, valid_from, None)
+    dno = participant.insert_party(sess, market_role_R, "WPD", vf, None, "22")
+    meter_type = MeterType.insert(sess, "C5", "COP 1-5", vf, None)
+    meter_payment_type = MeterPaymentType.insert(sess, "CR", "Credit", vf, None)
+    mtc = Mtc.insert(sess, "845", False, True, vf, None)
     mtc_participant = MtcParticipant.insert(
         sess,
         mtc,
@@ -439,37 +440,23 @@ def test_process_in_chellow_not_ecoes(mocker, sess):
         meter_type,
         meter_payment_type,
         0,
-        utc_datetime(1996, 1, 1),
+        vf,
         None,
     )
     insert_voltage_levels(sess)
     voltage_level = VoltageLevel.get_by_code(sess, "HV")
     llfc = dno.insert_llfc(
-        sess,
-        "510",
-        "PC 5-8 & HH HV",
-        voltage_level,
-        False,
-        True,
-        utc_datetime(1996, 1, 1),
-        None,
-    )
-    MtcLlfc.insert(sess, mtc_participant, llfc, valid_from, None)
-    dno.insert_llfc(
-        sess,
-        "521",
-        "Export (HV)",
-        voltage_level,
-        False,
-        False,
-        utc_datetime(1996, 1, 1),
-        None,
+        sess, "510", "PC 5-8 & HH HV", voltage_level, False, True, vf, None
     )
+    MtcLlfc.insert(sess, mtc_participant, llfc, vf, None)
+    dno.insert_llfc(sess, "521", "Export (HV)", voltage_level, False, False, vf, None)
     insert_sources(sess)
-    source = Source.get_by_code(sess, "net")
+    source = Source.get_by_code(sess, "grid")
     insert_energisation_statuses(sess)
     energisation_status = EnergisationStatus.get_by_code(sess, "E")
     gsp_group = GspGroup.insert(sess, "_L", "South Western")
+    insert_dtc_meter_types(sess)
+    dtc_meter_type = DtcMeterType.get_by_code(sess, "H")
 
     site.insert_e_supply(
         sess,
@@ -491,7 +478,7 @@ def test_process_in_chellow_not_ecoes(mocker, sess):
         comm,
         None,
         energisation_status,
-        {},
+        dtc_meter_type,
         "22 7868 6232 789",
         "510",
         imp_supplier_contract,
@@ -534,25 +521,30 @@ def test_process_in_chellow_not_ecoes(mocker, sess):
             "ecoes_pc",
             "chellow_pc",
             "ecoes_mtc",
+            "ecoes_mtc_date",
             "chellow_mtc",
             "ecoes_llfc",
+            "ecoes_llfc_from",
             "chellow_llfc",
             "ecoes_ssc",
             "chellow_ssc",
             "ecoes_es",
             "chellow_es",
             "ecoes_supplier",
+            "ecoes_supplier_registration_from",
             "chellow_supplier",
             "chellow_supplier_contract_name",
             "ecoes_dc",
             "chellow_dc",
             "ecoes_mop",
+            "ecoes_mop_appoint_date",
             "chellow_mop",
             "ecoes_gsp_group",
+            "ecoes_gsp_effective_from",
             "chellow_gsp_group",
             "ecoes_msn",
-            "chellow_msn",
             "ecoes_msn_install_date",
+            "chellow_msn",
             "ecoes_meter_type",
             "chellow_meter_type",
             "ignored",
@@ -564,103 +556,73 @@ def test_process_in_chellow_not_ecoes(mocker, sess):
             "",
             "00",
             "",
+            "",
             "845",
             "",
+            "",
             "510",
             "",
             "",
             "",
             "E",
             "",
+            "",
             "CALB",
             "Fusion Supplier 2000",
             "",
             "CALB",
             "",
+            "",
             "CALB",
             "",
+            "",
             "_L",
             "",
-            "hgjeyhuw",
             "",
+            "hgjeyhuw",
             "",
             "H",
             "False",
-            '"In Chellow, but not in ECOES."',
+            "In Chellow, but not in ECOES.",
         ],
     ]
-    assert f.getvalue() == "\n".join(",".join(line) for line in expected) + "\n"
+    f.seek(0)
+    actual = list(csv.reader(f))
+    match_tables(expected, actual)
 
 
-def test_meter_type(mocker, sess):
-    valid_from = to_utc(ct_datetime(1996, 1, 1))
+def test_process_dtc_none(mocker, sess):
+    vf = to_utc(ct_datetime(2000, 1, 1))
     site = Site.insert(sess, "CI017", "Water Works")
 
     market_role_Z = MarketRole.insert(sess, "Z", "Non-core")
     participant = Participant.insert(sess, "CALB", "AK Industries")
-    participant.insert_party(
-        sess, market_role_Z, "None core", utc_datetime(2000, 1, 1), None, None
-    )
-    bank_holiday_rate_script = {"bank_holidays": []}
-    Contract.insert_non_core(
-        sess,
-        "bank_holidays",
-        "",
-        {},
-        utc_datetime(2000, 1, 1),
-        None,
-        bank_holiday_rate_script,
-    )
+    participant.insert_party(sess, market_role_Z, "None core", vf, None, None)
     market_role_X = MarketRole.insert(sess, "X", "Supplier")
     market_role_M = MarketRole.insert(sess, "M", "Mop")
     market_role_C = MarketRole.insert(sess, "C", "HH Dc")
     market_role_R = MarketRole.insert(sess, "R", "Distributor")
-    participant.insert_party(
-        sess, market_role_M, "Fusion Mop Ltd", utc_datetime(2000, 1, 1), None, None
-    )
-    participant.insert_party(
-        sess, market_role_X, "Fusion Ltc", utc_datetime(2000, 1, 1), None, None
-    )
-    participant.insert_party(
-        sess, market_role_C, "Fusion DC", utc_datetime(2000, 1, 1), None, None
-    )
+    participant.insert_party(sess, market_role_M, "Fusion Mop", vf, None, None)
+    participant.insert_party(sess, market_role_X, "Fusion Ltc", vf, None, None)
+    participant.insert_party(sess, market_role_C, "Fusion DC", vf, None, None)
     mop_contract = Contract.insert_mop(
-        sess, "Fusion", participant, "", {}, utc_datetime(2000, 1, 1), None, {}
+        sess, "Fusion", participant, "", {}, vf, None, {}
     )
     dc_contract = Contract.insert_dc(
-        sess, "Fusion DC 2000", participant, "", {}, utc_datetime(2000, 1, 1), None, {}
+        sess, "Fusion DC 2000", participant, "", {}, vf, None, {}
     )
-    pc = Pc.insert(sess, "02", "nhh", utc_datetime(2000, 1, 1), None)
+    pc = Pc.insert(sess, "00", "hh", vf, None)
     insert_cops(sess)
     cop = Cop.get_by_code(sess, "5")
-    ssc = Ssc.insert(sess, "0393", "unrestricted", True, utc_datetime(2000, 1), None)
     insert_comms(sess)
     comm = Comm.get_by_code(sess, "GSM")
     imp_supplier_contract = Contract.insert_supplier(
-        sess,
-        "Fusion Supplier 2000",
-        participant,
-        "",
-        {},
-        utc_datetime(2000, 1, 1),
-        None,
-        {},
-    )
-    dno = participant.insert_party(
-        sess, market_role_R, "WPD", utc_datetime(2000, 1, 1), None, "22"
-    )
-    meter_type = MeterType.insert(sess, "C5", "COP 1-5", utc_datetime(2000, 1, 1), None)
-    meter_payment_type = MeterPaymentType.insert(
-        sess, "CR", "Credit", utc_datetime(1996, 1, 1), None
-    )
-    mtc = Mtc.insert(
-        sess,
-        "845",
-        False,
-        True,
-        utc_datetime(1996, 1, 1),
-        None,
+        sess, "Fusion Supplier 2000", participant, "", {}, vf, None, {}
     )
+    dno = participant.insert_party(sess, market_role_R, "WPD", vf, None, "22")
+    meter_type = MeterType.insert(sess, "C5", "COP 1-5", vf, None)
+    meter_payment_type = MeterPaymentType.insert(sess, "CR", "Credit", vf, None)
+    mtc = Mtc.insert(sess, "845", False, True, vf, None)
     mtc_participant = MtcParticipant.insert(
         sess,
         mtc,
@@ -671,32 +633,23 @@ def test_meter_type(mocker, sess):
         meter_type,
         meter_payment_type,
         0,
-        utc_datetime(1996, 1, 1),
+        vf,
         None,
     )
     insert_voltage_levels(sess)
     voltage_level = VoltageLevel.get_by_code(sess, "HV")
     llfc = dno.insert_llfc(
-        sess,
-        "510",
-        "PC 5-8 & HH HV",
-        voltage_level,
-        False,
-        True,
-        utc_datetime(1996, 1, 1),
-        None,
+        sess, "510", "PC 5-8 & HH HV", voltage_level, False, True, vf, None
     )
+    MtcLlfc.insert(sess, mtc_participant, llfc, vf, None)
+    dno.insert_llfc(sess, "521", "Export (HV)", voltage_level, False, False, vf, None)
     insert_sources(sess)
-    source = Source.get_by_code(sess, "net")
+    source = Source.get_by_code(sess, "grid")
     insert_energisation_statuses(sess)
     energisation_status = EnergisationStatus.get_by_code(sess, "E")
     gsp_group = GspGroup.insert(sess, "_L", "South Western")
-    MtcLlfc.insert(sess, mtc_participant, llfc, valid_from, None)
-    mtc_ssc = MtcSsc.insert(sess, mtc_participant, ssc, valid_from, None)
-    mtc_llfc_ssc = MtcLlfcSsc.insert(sess, mtc_ssc, llfc, valid_from, None)
-    MtcLlfcSscPc.insert(sess, mtc_llfc_ssc, pc, valid_from, None)
 
-    supply = site.insert_e_supply(
+    site.insert_e_supply(
         sess,
         source,
         None,
@@ -714,9 +667,9 @@ def test_meter_type(mocker, sess):
         "845",
         cop,
         comm,
-        ssc.code,
+        None,
         energisation_status,
-        {},
+        None,
         "22 7868 6232 789",
         "510",
         imp_supplier_contract,
@@ -728,9 +681,105 @@ def test_meter_type(mocker, sess):
         None,
         None,
     )
-
-    era = supply.eras[0]
-
-    _meter_type(
-        era,
+    report_run = ReportRun.insert(
+        sess,
+        "ecoes_comparison",
+        None,
+        "ecoes_comparison",
+        {},
     )
+
+    sess.commit()
+    f = StringIO()
+    ecoes_lines = [
+        "titles",
+        ",".join(
+            (
+                "2278686232789",
+                "address-line-1",
+                "address-line-2",
+                "address-line-3",
+                "address-line-4",
+                "address-line-5",
+                "address-line-6",
+                "address-line-7",
+                "address-line-8",
+                "address-line-9",
+                "post-code",
+                "CALB",
+                "12/10/2022",
+                "845",
+                "12/10/2022",
+                "510",
+                "12/10/2022",
+                "00",
+                "",
+                "measurement-class",
+                "E",
+                "CALB",
+                "CALB",
+                "CALB",
+                "12/10/2022",
+                "_L",
+                "12/10/2022",
+                "22",
+                "hgjeyhuw",
+                "12/10/2022",
+                "",
+                "map-id",
+            ),
+        ),
+    ]
+    exclude_mpan_cores = []
+    ignore_mpan_cores_msn = []
+    show_ignored = True
+
+    _process(
+        sess,
+        ecoes_lines,
+        exclude_mpan_cores,
+        ignore_mpan_cores_msn,
+        f,
+        show_ignored,
+        report_run,
+    )
+    expected = [
+        [
+            "mpan_core",
+            "mpan_core_no_spaces",
+            "ecoes_pc",
+            "chellow_pc",
+            "ecoes_mtc",
+            "ecoes_mtc_date",
+            "chellow_mtc",
+            "ecoes_llfc",
+            "ecoes_llfc_from",
+            "chellow_llfc",
+            "ecoes_ssc",
+            "chellow_ssc",
+            "ecoes_es",
+            "chellow_es",
+            "ecoes_supplier",
+            "ecoes_supplier_registration_from",
+            "chellow_supplier",
+            "chellow_supplier_contract_name",
+            "ecoes_dc",
+            "chellow_dc",
+            "ecoes_mop",
+            "ecoes_mop_appoint_date",
+            "chellow_mop",
+            "ecoes_gsp_group",
+            "ecoes_gsp_effective_from",
+            "chellow_gsp_group",
+            "ecoes_msn",
+            "ecoes_msn_install_date",
+            "chellow_msn",
+            "ecoes_meter_type",
+            "chellow_meter_type",
+            "ignored",
+            "problem",
+        ],
+    ]
+    f.seek(0)
+    actual = list(csv.reader(f))
+    match_tables(expected, actual)
